@@ -78,17 +78,25 @@ extern BIO *bio_err;
 
 #endif
 
+#include <signal.h>
+
+#ifdef SIGPIPE
+#define do_pipe_sig()	signal(SIGPIPE,SIG_IGN)
+#else
+#define do_pipe_sig()
+#endif
+
 #if defined(MONOLITH) && !defined(SSLEAY)
-#  define apps_startup()
+#  define apps_startup()	do_pipe_sig()
 #else
 #  if defined(MSDOS) || defined(WIN16) || defined(WIN32)
 #    ifdef _O_BINARY
-#      define apps_startup() _fmode=_O_BINARY
+#      define apps_startup()	_fmode=_O_BINARY; do_pipe_sig()
 #    else
-#      define apps_startup() _fmode=O_BINARY
+#      define apps_startup()	_fmode=O_BINARY; do_pipe_sig()
 #    endif
 #  else
-#    define apps_startup()
+#    define apps_startup()	do_pipe_sig()
 #  endif
 #endif
 
