@@ -44,6 +44,7 @@
  * copied and put under another distribution licence
  * [including the GNU Public Licence.]
  */
+/* The PPKI stuff has been donated by Jeff Barber <jeffb@issl.atl.hp.com> */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -123,6 +124,7 @@ static char *ca_usage[]={
 " -out file       - Where to put the output file(s)\n",
 " -outdir dir     - Where to put output certificates\n",
 " -infiles ....   - The last argument, requests to process\n",
+" -spkac file     - File contains DN and signed public key and challenge\n",
 NULL
 };
 
@@ -139,8 +141,15 @@ static int save_serial(char *serialfile, BIGNUM *serial);
 static int certify(X509 **xret, char *infile,RSA *rsa,X509 *x509,
 	EVP_MD *dgst,STACK *policy,TXT_DB *db,BIGNUM *serial,int days,
 	int batch, int verbose);
+static int certify_spkac(X509 **xret, char *infile,RSA *rsa,X509 *x509,
+	EVP_MD *dgst,STACK *policy,TXT_DB *db,BIGNUM *serial,int days,
+	int verbose);
+static int fix_data(int nid, int *type);
 static int cmp_on_nid(X509_NAME_ENTRY **a,X509_NAME_ENTRY **b);
-static void write_new_certificate(BIO *bp, X509 *x);
+static void write_new_certificate(BIO *bp, X509 *x, int output_der);
+static int do_body(X509 **xret, RSA *rsa, X509 *x509, EVP_MD *dgst,
+	STACK *policy, TXT_DB *db, BIGNUM *serial, int days, int batch,
+	int verbose, X509_REQ *req);
 #else
 static void lookup_fail();
 static int MS_CALLBACK key_callback();
@@ -154,6 +163,7 @@ static int save_serial();
 static int certify();
 static int cmp_on_nid();
 static void write_new_certificate();
+static int do_body();
 #endif
 
 static LHASH *conf;
@@ -173,13 +183,15 @@ char **argv;
 	int gencrl=0;
 	long crldays=0;
 	long crlhours=0;
-	long errorline=-1;
+	long errorline= -1;
 	char *configfile=NULL;
 	char *md=NULL;
 	char *policy=NULL;
 	char *keyfile=NULL;
 	char *certfile=NULL;
 	char *infile=NULL;
+	char *spkac_file=NULL;
+	int output_der = 0;
 	char *outfile=NULL;
 	char *outdir=NULL;
 	char *serialfile=NULL;
@@ -294,6 +306,12 @@ char **argv;
 			req=1;
 			break;
 			}
+		else if (strcmp(*argv, "-spkac") == 0)
+			{
+			if (--argc < 1) goto bad;
+			spkac_file = *(++argv);
+			req=1;
+			}
 		else
 			{
 bad:
@@ -601,6 +619,29 @@ bad:
 			fprintf(stderr,"malloc failure\n");
 			goto err;
 			}
+		if (spkac_file != NULL)
+			{
+			total++;
+			j=certify_spkac(&x,spkac_file,rsa,x509,dgst,attribs,db,
+				serial,days,verbose);
+			if (j < 0) goto err;
+			if (j > 0)
+				{
+				total_done++;
+				fprintf(stderr,"\n");
+				if (!BN_add_word(serial,1)) goto err;
+				if (!sk_push(cert_sk,(char *)x))
+					{
+					fprintf(stderr,"malloc failure\n");
+					goto err;
+					}
+				if (outfile)
+					{
+					output_der = 1;
+					batch = 1;
+					}
+				}
+			}
 		if (infile != NULL)
 			{
 			total++;
@@ -714,8 +755,8 @@ bad:
 				perror(buf[2]);
 				goto err;
 				}
-			write_new_certificate(Cout,x);
-			write_new_certificate(Sout,x);
+			write_new_certificate(Cout,x, 0);
+			write_new_certificate(Sout,x, output_der);
 			}
 
 		if (sk_num(cert_sk))
@@ -1021,32 +1062,12 @@ int days;
 int batch;
 int verbose;
 	{
-	BIO *in=NULL,*b_err=NULL;
-	STACK *name=NULL,*CAname=NULL;
-	STACK *subject=NULL;
 	X509_REQ *req=NULL;
-	X509 *ret=NULL;
-	X509_CINF *ci;
-	X509_NAME_ENTRY *ne;
-	X509_NAME_ENTRY tmp,*tne,*push;
-	int ok=-1,i,j;
-	char *p;
-	CONF_VALUE *cv;
-	char *row[DB_NUMBER],**rrow,**irow=NULL;
+	BIO *in=NULL;
 	EVP_PKEY *pkey=NULL;
-	EVP_PKEY static_pkey;
-
-	for (i=0; i<DB_NUMBER; i++)
-		row[i]=NULL;
+	int ok= -1,i;
 
 	in=BIO_new(BIO_s_file());
-	b_err=BIO_new(BIO_s_file());
-	if ((in == NULL) || (b_err == NULL))
-		{
-		ERR_print_errors(bio_err);
-		goto err;
-		}
-	BIO_set_fp(b_err,stderr,BIO_NOCLOSE);
 
 	if (BIO_read_filename(in,infile) <= 0)
 		{
@@ -1060,7 +1081,7 @@ int verbose;
 		goto err;
 		}
 	if (verbose)
-		X509_REQ_print(b_err,req);
+		X509_REQ_print(bio_err,req);
 
 	fprintf(stderr,"Check that the request matches the signature\n");
 
@@ -1080,7 +1101,13 @@ int verbose;
 		}
 	i=X509_REQ_verify(req,pkey);
 	EVP_PKEY_free(pkey);
-	if (!i)
+	if (i < 0)
+		{
+		ok=0;
+		fprintf(stderr,"Signature verification problems....\n");
+		goto err;
+		}
+	if (i == 0)
 		{
 		ok=0;
 		fprintf(stderr,"Signature did not match the certificate request\n");
@@ -1089,12 +1116,48 @@ int verbose;
 	else
 		fprintf(stderr,"Signature ok\n");
 
+	ok=do_body(xret,rsa,x509,dgst,policy,db,serial,days,batch,verbose,req);
+
+err:
+	if (req != NULL) X509_REQ_free(req);
+	if (in != NULL) BIO_free(in);
+	return(ok);
+	}
+
+static int do_body(xret,rsa,x509,dgst,policy,db,serial,days,batch,verbose,req)
+X509 **xret;
+RSA *rsa;
+X509 *x509;
+EVP_MD *dgst;
+STACK *policy;
+TXT_DB *db;
+BIGNUM *serial;
+int days;
+int batch;
+int verbose;
+X509_REQ *req;
+	{
+	STACK *name=NULL,*CAname=NULL;
+	STACK *subject=NULL;
+	X509 *ret=NULL;
+	X509_CINF *ci;
+	X509_NAME_ENTRY *ne;
+	X509_NAME_ENTRY tmp,*tne,*push;
+	int ok= -1,i,j;
+	char *p;
+	CONF_VALUE *cv;
+	char *row[DB_NUMBER],**rrow,**irow=NULL;
+	EVP_PKEY static_pkey;
+
+	for (i=0; i<DB_NUMBER; i++)
+		row[i]=NULL;
+
 	fprintf(stderr,"The Subjects Distinguished Name is as follows\n");
 	name=req->req_info->subject;
 	for (i=0; i<sk_num(name); i++)
 		{
 		ne=(X509_NAME_ENTRY *)sk_value(name,i);
-		j=i2a_ASN1_OBJECT(b_err,ne->object);
+		j=i2a_ASN1_OBJECT(bio_err,ne->object);
 		for (j=22-j; j>0; j--)
 			fputc(' ',stderr);
 		fputc(':',stderr);
@@ -1120,10 +1183,9 @@ int verbose;
 			((j == V_ASN1_IA5STRING) &&
 			 (ne->value->type == V_ASN1_PRINTABLESTRING)))
 			{
-			fprintf(stderr,"\nThe string contains characters that are ilegal for the ASN.1 type\n");
+			fprintf(stderr,"\nThe string contains characters that are illegal for the ASN.1 type\n");
 			goto err;
 			}
-
 			
 		p=(char *)ne->value->data;
 		for (j=ne->value->length; j>0; j--)
@@ -1190,13 +1252,13 @@ int verbose;
 			{
 			if (tne == NULL)
 				{
-				fprintf(stderr,"The manditory %s field was missing\n",cv->name);
+				fprintf(stderr,"The mandatory %s field was missing\n",cv->name);
 				goto err;
 				}
 			j=sk_find(CAname,(char *)&tmp);
 			if (j < 0)
 				{
-				fprintf(stderr,"The %s field does not exist in the CA certificate,\nthe 'policy' is missconfigured\n",cv->name);
+				fprintf(stderr,"The %s field does not exist in the CA certificate,\nthe 'policy' is misconfigured\n",cv->name);
 				goto err;
 				}
 
@@ -1241,7 +1303,7 @@ int verbose;
 	row[DB_serial]=BN_bn2ascii(serial);
 	if ((row[DB_name] == NULL) || (row[DB_serial] == NULL))
 		{
-		fprintf(stderr,"malloc failur\n");
+		fprintf(stderr,"malloc failure\n");
 		goto err;
 		}
 
@@ -1298,20 +1360,14 @@ int verbose;
 
 	if ((ret=X509_new()) == NULL) goto err;
 	ci=ret->cert_info;
-/*#ifdef undef */
-	if ((req->req_info->attributes != NULL) &&
-		(sk_num(req->req_info->attributes) > 0))
-		{
-		if (ci->version == NULL)
-			if ((ci->version=ASN1_INTEGER_new()) == NULL)
-				goto err;
-		ASN1_INTEGER_set(ci->version,2);
-		j=1;
-		}
-	else
-/* #endif */
-		j=0;
-	/* else ci->version=NULL; */
+
+#ifdef X509_V3
+	/* Make it an X509 v3 certificate. */
+	if (ci->version == NULL)
+		if ((ci->version=ASN1_INTEGER_new()) == NULL)
+			goto err;
+	ASN1_INTEGER_set(ci->version,2);
+#endif
 
 	if (BN_to_ASN1_INTEGER(serial,ci->serialNumber) == NULL)
 		goto err;
@@ -1322,7 +1378,7 @@ int verbose;
 	fprintf(stderr,"Certificate is to be certified until ");
 	X509_gmtime_adj(ci->validity->notBefore,0);
 	X509_gmtime_adj(ci->validity->notAfter,(long)60*60*24*days);
-	ASN1_UTCTIME_print(b_err,ci->validity->notAfter);
+	ASN1_UTCTIME_print(bio_err,ci->validity->notAfter);
 	fprintf(stderr," (%d days)\n",days);
 
 	X509_NAME_free(ci->subject);
@@ -1333,14 +1389,6 @@ int verbose;
 	ci->key=req->req_info->pubkey;
 	req->req_info->pubkey=NULL;
 
-/* #ifdef 1 */
-	if (j)
-		{
-		ci->attributes=req->req_info->attributes;
-		req->req_info->attributes=NULL;
-		}
-/* #endif */
-
 	if (!batch)
 		{
 		char buf[10];
@@ -1403,9 +1451,6 @@ err:
 	for (i=0; i<DB_NUMBER; i++)
 		if (row[i] != NULL) free(row[i]);
 
-	if (req != NULL) X509_REQ_free(req);
-	if (in != NULL) BIO_free(in);
-	if (b_err != NULL) BIO_free(b_err);
 	if (CAname != NULL)
 		X509_NAME_free(CAname);
 	if (ok <= 0)
@@ -1433,12 +1478,19 @@ X509_NAME_ENTRY **a,**b;
 	return(OBJ_obj2nid((*a)->object)-OBJ_obj2nid((*b)->object));
 	}
 
-static void write_new_certificate(bp,x)
+static void write_new_certificate(bp,x, output_der)
 BIO *bp;
 X509 *x;
+int output_der;
 	{
 	char *f;
 
+	if (output_der)
+	{
+	    (void)i2d_X509_bio(bp,x);
+	    return;
+	}
+
 	f=X509_NAME_oneline(X509_get_issuer_name(x));
 	BIO_puts(bp,"issuer :");
 	BIO_puts(bp,f);
@@ -1456,3 +1508,203 @@ X509 *x;
 	BIO_puts(bp,"\n");
 	}
 
+static int certify_spkac(xret,infile,rsa,x509,dgst,policy,db,serial,days,
+	verbose)
+X509 **xret;
+char *infile;
+RSA *rsa;
+X509 *x509;
+EVP_MD *dgst;
+STACK *policy;
+TXT_DB *db;
+BIGNUM *serial;
+int days;
+int verbose;
+	{
+	STACK *sk=NULL;
+	LHASH *parms=NULL;
+	X509_REQ *req=NULL;
+	CONF_VALUE *cv=NULL;
+	NETSCAPE_SPKI *spki = NULL;
+	unsigned char *spki_der = NULL,*p;
+	X509_REQ_INFO *ri;
+	char *type;
+	EVP_PKEY *pkey=NULL;
+	X509_NAME *n=NULL;
+	X509_NAME_ENTRY *ne=NULL;
+	int ok= -1,i,j;
+	long errline;
+	int nid;
+
+	/*
+	 * Load input file into a hash table.  (This is just an easy
+	 * way to read and parse the file, then put it into a convenient
+	 * STACK format).
+	 */
+	parms=CONF_load(NULL,infile,&errline);
+	if (parms == NULL)
+		{
+		fprintf(stderr,"error on line %ld of %s\n",errline,infile);
+		ERR_print_errors(bio_err);
+		goto err;
+		}
+
+	sk=CONF_get_section(parms, "default");
+	if (sk_num(sk) == 0)
+		{
+		fprintf(stderr, "no name/value pairs found in %s\n", infile);
+		CONF_free(parms);
+		goto err;
+		}
+
+	/*
+	 * Now create a dummy X509 request structure.  We don't actually
+	 * have an X509 request, but we have many of the components
+	 * (a public key, various DN components).  The idea is that we
+	 * put these components into the right X509 request structure
+	 * and we can use the same code as if you had a real X509 request.
+	 */
+	req=X509_REQ_new();
+	if (req == NULL)
+		{
+		ERR_print_errors(bio_err);
+		goto err;
+		}
+
+	/*
+	 * Build up the subject name set.
+	 */
+	ri=req->req_info;
+	n = ri->subject;
+
+	for (i = 0; ; i++)
+		{
+		if ((int)sk_num(sk) <= i) break;
+
+		cv=(CONF_VALUE *)sk_value(sk,i);
+		type=cv->name;
+
+		if ((nid=OBJ_txt2nid(type)) == NID_undef)
+			{
+			if (strcmp(type, "SPKAC") == 0)
+				{
+				spki_der = malloc(strlen(cv->value) + 1);
+				if (spki_der == NULL)
+					{
+					fprintf(stderr,"malloc failure\n");
+					goto err;
+					}
+				j = EVP_DecodeBlock(spki_der, (unsigned char *)cv->value,
+					strlen(cv->value));
+				if (j <= 0)
+					{
+					fprintf(stderr, "Can't b64 decode SPKAC structure\n");
+					goto err;
+					}
+
+				p=spki_der;
+				spki = d2i_NETSCAPE_SPKI(&spki, &p, j);
+				free(spki_der);
+				spki_der = NULL;
+				if (spki == NULL)
+					{
+					fprintf(stderr,"unable to load Netscape SPKAC structure\n");
+					ERR_print_errors(bio_err);
+					goto err;
+					}
+				}
+			continue;
+			}
+
+		/* add object plus value */
+		j = sk_num(n);
+		if (j) j=((X509_NAME_ENTRY *)(sk_value(n, j-1)))->set+1;
+
+		if ((ne=X509_NAME_ENTRY_new()) == NULL) goto err;
+
+		ne->object=OBJ_nid2obj(nid);
+		ne->set=j;
+
+		ne->value->type=ASN1_PRINTABLE_type((unsigned char *)cv->value);
+
+		if (fix_data(nid, &ne->value->type) == 0)
+			{
+			fprintf(stderr, "invalid characters in string %s\n",
+				cv->value);
+			goto err;
+			}
+
+		j = strlen(cv->value);
+		ne->value->length=j;
+		ne->value->data=(unsigned char *)malloc(j+1);
+		if (ne->value->data == NULL)
+			{ fprintf(stderr,"malloc failure\n"); goto err; }
+		memcpy(ne->value->data,cv->value,j+1);
+		if (!sk_push(n,(char *)ne)) goto err;
+		ne=NULL;
+		}
+	if (spki == NULL)
+		{
+		fprintf(stderr,"Netscape SPKAC structure not found in %s\n",
+			infile);
+		goto err;
+		}
+
+	/*
+	 * Now extract the key from the SPKI structure.
+	 */
+
+	fprintf(stderr,"Check that the SPKAC request matches the signature\n");
+
+	nid=OBJ_obj2nid(spki->spkac->pubkey->algor->algorithm);
+	p=spki->spkac->pubkey->public_key->data;
+	j=spki->spkac->pubkey->public_key->length;
+	pkey=d2i_PublicKey(nid,NULL,&p,(long)j);
+	if (pkey == NULL)
+		{
+		fprintf(stderr,"error unpacking SPKAC public key\n");
+		goto err;
+		}
+
+	j = NETSCAPE_SPKI_verify(spki, pkey);
+	EVP_PKEY_free(pkey);
+	if (j <= 0)
+		{
+		fprintf(stderr,"signature verification failed on SPKAC public key\n");
+		goto err;
+		}
+	fprintf(stderr,"Signature ok\n");
+
+	
+	X509_PUBKEY_free(req->req_info->pubkey);
+	req->req_info->pubkey=spki->spkac->pubkey;
+	spki->spkac->pubkey=NULL;
+	ok=do_body(xret,rsa,x509,dgst,policy,db,serial,days,1,verbose,req);
+err:
+	if (req != NULL) X509_REQ_free(req);
+	if (parms != NULL) CONF_free(parms);
+	if (spki_der != NULL) free(spki_der);
+	if (spki != NULL) NETSCAPE_SPKI_free(spki);
+	if (ne != NULL) X509_NAME_ENTRY_free(ne);
+
+	return(ok);
+	}
+
+static int fix_data(nid,type)
+int nid;
+int *type;
+	{
+	if (nid == NID_pkcs9_emailAddress)
+		*type=V_ASN1_IA5STRING;
+	if ((nid == NID_commonName) && (*type == V_ASN1_IA5STRING))
+		*type=V_ASN1_T61STRING;
+	if ((nid == NID_pkcs9_challengePassword) && (*type == V_ASN1_IA5STRING))
+		*type=V_ASN1_T61STRING;
+	if ((nid == NID_pkcs9_unstructuredName) && (*type == V_ASN1_T61STRING))
+		return(0);
+	if (nid == NID_pkcs9_unstructuredName)
+		*type=V_ASN1_IA5STRING;
+	return(1);
+	}
+
+
