@@ -46,10 +46,12 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include "apps.h"
 #include "err.h"
+#include "objects.h"
 #include "envelope.h"
 #include "x509.h"
 #include "pkcs7.h"
@@ -77,6 +79,7 @@ char **argv;
 	BIO *in=NULL,*out=NULL;
 	int informat,outformat;
 	char *infile,*outfile,*prog;
+	int print_certs=0;
 	int ret=0;
 
 	apps_startup();
@@ -115,6 +118,8 @@ char **argv;
 			if (--argc < 1) goto bad;
 			outfile= *(++argv);
 			}
+		else if (strcmp(*argv,"-print_certs") == 0)
+			print_certs=1;
 		else if (strcmp(*argv,"-des") == 0)
 			enc=EVP_des_cbc();
 		else if (strcmp(*argv,"-des3") == 0)
@@ -142,6 +147,7 @@ bad:
 		fprintf(stderr," -outform arg  output format - one of DER TXT PEM\n");
 		fprintf(stderr," -in arg       inout file\n");
 		fprintf(stderr," -out arg      output file\n");
+		fprintf(stderr," -print_certs  print any certs or crl in the input\n");
 		fprintf(stderr," -des          encrypt PEM output with cbc des\n");
 		fprintf(stderr," -des3         encrypt PEM output with ede cbc des using 168 bit key\n");
 #ifndef NO_IDEA
@@ -199,6 +205,80 @@ bad:
 			}
 		}
 
+	if (print_certs)
+		{
+		int i;
+		STACK *certs=NULL;
+		STACK *crls=NULL;
+
+		i=OBJ_obj2nid(p7->type);
+		switch (i)
+			{
+		case NID_pkcs7_signed:
+			certs=p7->d.sign->cert;
+			crls=p7->d.sign->crl;
+			break;
+		case NID_pkcs7_signedAndEnveloped:
+			certs=p7->d.signed_and_enveloped->cert;
+			crls=p7->d.signed_and_enveloped->crl;
+			break;
+		default:
+			break;
+			}
+
+		if (certs != NULL)
+			{
+			char *a;
+			X509 *x;
+
+			for (i=0; i<sk_num(certs); i++)
+				{
+				x=(X509 *)sk_value(certs,i);
+
+				a=X509_NAME_oneline(X509_get_subject_name(x));
+				BIO_puts(out,"subject=");
+				BIO_puts(out,a);
+				free(a);
+
+				a=X509_NAME_oneline(X509_get_issuer_name(x));
+				BIO_puts(out,"\nissuer= ");
+				BIO_puts(out,a);
+				BIO_puts(out,"\n");
+				free(a);
+
+				PEM_write_bio_X509(out,x);
+				BIO_puts(out,"\n");
+				}
+			}
+		if (crls != NULL)
+			{
+			X509_CRL *crl;
+			char *a;
+
+			for (i=0; i<sk_num(crls); i++)
+				{
+				crl=(X509_CRL *)sk_value(crls,i);
+
+				a=X509_NAME_oneline(crl->crl->issuer);
+				BIO_puts(out,"issuer= ");
+				BIO_puts(out,a);
+				free(a);
+
+				BIO_puts(out,"\nlast update=");
+				ASN1_UTCTIME_print(out,crl->crl->lastUpdate);
+				BIO_puts(out,"\nnext update=");
+				ASN1_UTCTIME_print(out,crl->crl->nextUpdate);
+				BIO_puts(out,"\n");
+
+				PEM_write_bio_X509_CRL(out,crl);
+				BIO_puts(out,"\n");
+				}
+			}
+
+		ret=0;
+		goto end;
+		}
+
 	if 	(outformat == FORMAT_ASN1)
 		i=i2d_PKCS7_bio(out,p7);
 	else if (outformat == FORMAT_PEM)
