@@ -22,6 +22,7 @@ extern int s_mult_main(int argc,char *argv[]);
 extern int version_main(int argc,char *argv[]);
 extern int pkcs7_main(int argc,char *argv[]);
 extern int crl2pkcs7_main(int argc,char *argv[]);
+extern int sess_id_main(int argc,char *argv[]);
 #else
 extern int verify_main();
 extern int asn1parse_main();
@@ -46,6 +47,7 @@ extern int s_mult_main();
 extern int version_main();
 extern int pkcs7_main();
 extern int crl2pkcs7_main();
+extern int sess_id_main();
 #endif
 
 #define FUNC_TYPE_GENERAL	1
@@ -92,6 +94,7 @@ FUNCTION functions[] = {
 	{FUNC_TYPE_GENERAL,"version",version_main},
 	{FUNC_TYPE_GENERAL,"pkcs7",pkcs7_main},
 	{FUNC_TYPE_GENERAL,"crl2pkcs7",crl2pkcs7_main},
+	{FUNC_TYPE_GENERAL,"sess_id",sess_id_main},
 	{FUNC_TYPE_MD,"md2",dgst_main},
 	{FUNC_TYPE_MD,"md5",dgst_main},
 	{FUNC_TYPE_MD,"sha",dgst_main},
@@ -99,6 +102,7 @@ FUNCTION functions[] = {
 	{FUNC_TYPE_CIPHER,"base64",enc_main},
 	{FUNC_TYPE_CIPHER,"des",enc_main},
 	{FUNC_TYPE_CIPHER,"des3",enc_main},
+	{FUNC_TYPE_CIPHER,"desx",enc_main},
 #ifndef NO_IDEA
 	{FUNC_TYPE_CIPHER,"idea",enc_main},
 #endif
