@@ -68,6 +68,7 @@
  * -des3	- encrypt output if PEM format
  * -idea	- encrypt output if PEM format
  * -text	- print a text version
+ * -modulus	- print the RSA key modulus
  */
 
 int MAIN(argc, argv)
@@ -81,6 +82,7 @@ char **argv;
 	BIO *in=NULL,*out=NULL;
 	int informat,outformat,text=0,noout=0;
 	char *infile,*outfile,*prog;
+	int modulus=0;
 
 	apps_startup();
 
@@ -122,6 +124,8 @@ char **argv;
 			noout=1;
 		else if (strcmp(*argv,"-text") == 0)
 			text=1;
+		else if (strcmp(*argv,"-modulus") == 0)
+			modulus=1;
 		else if (strcmp(*argv,"-des") == 0)
 			enc=EVP_des_cbc();
 		else if (strcmp(*argv,"-des3") == 0)
@@ -156,6 +160,7 @@ bad:
 #endif
 		fprintf(stderr," -text         print the key in text\n");
 		fprintf(stderr," -noout        don't print key out\n");
+		fprintf(stderr," -modulus      print the RSA key modulus\n");
 		goto end;
 		}
 
@@ -243,6 +248,13 @@ bad:
 			goto end;
 			}
 
+	if (modulus)
+		{
+		fprintf(stdout,"Modulus=");
+		BN_print(out,rsa->n);
+		fprintf(stdout,"\n");
+		}
+
 	if (noout) goto end;
 	fprintf(stderr,"writing RSA private key\n");
 	if 	(outformat == FORMAT_ASN1)
