@@ -229,6 +229,14 @@ bad:
 		}
 
 	if (state) SSL_CTX_set_info_callback(ctx,client_info_callback);
+	if (cipher == NULL)
+		SSL_CTX_set_cipher_list(ctx,getenv("SSL_CIPHER"));
+	else
+		SSL_CTX_set_cipher_list(ctx,cipher);
+
+	SSL_CTX_set_default_verify(ctx,verify,verify_callback);
+	if (!set_cert_stuff(ctx,cert_file,key_file))
+		goto end;
 
 	SSL_load_error_strings();
 
@@ -259,18 +267,6 @@ bad:
 	con=(SSL *)SSL_new(ctx);
 	if (c_Pause & 0x01) con->debug=1;
 	SSL_set_fd(con,s);
-	SSL_set_verify(con,verify,verify_callback);
-
-	if (cipher == NULL)
-		SSL_CTX_set_cipher_list(ctx,getenv("SSL_CIPHER"));
-	else
-		SSL_CTX_set_cipher_list(ctx,cipher);
-
-	if (!set_cert_stuff(con,cert_file,key_file))
-		{
-		SHUTDOWN(SSL_get_fd(con));
-		goto end;
-		}
 
 	/* ok, lets connect */
 	for (;;)
@@ -334,11 +330,6 @@ bad:
 			if (c_Pause & 0x01) con->debug=1;
 			SSL_set_fd(con2,s);
 			SSL_set_session(con2,SSL_get_session(con));
-			if (!set_cert_stuff(con2,cert_file,key_file))
-				{
-				SHUTDOWN(SSL_get_fd(con2));
-				goto end;
-				}
 			SSL_free(con);
 			con=con2;
 			con2=NULL;
