@@ -73,7 +73,7 @@ static void print_stats();
 
 #undef BUFSIZZ
 #define BUFSIZZ	8*1024
-static int accept_socket=-1;
+static int accept_socket= -1;
 
 #define TEST_CERT	"server.pem"
 #undef PROG
@@ -154,14 +154,15 @@ char *argv[];
 			}
 		else if	(strcmp(*argv,"-verify") == 0)
 			{
-			verify=SSL_VERIFY_PEER;
+			verify=SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE;
 			if (--argc < 1) goto bad;
 			verify_depth=atoi(*(++argv));
 			fprintf(stderr,"verify depth is %d\n",verify_depth);
 			}
 		else if	(strcmp(*argv,"-Verify") == 0)
 			{
-			verify=SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
+			verify=SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT|
+				SSL_VERIFY_CLIENT_ONCE;
 			if (--argc < 1) goto bad;
 			verify_depth=atoi(*(++argv));
 			fprintf(stderr,"verify depth is %d, must return a certificate\n",verify_depth);
@@ -249,17 +250,17 @@ SSL_CTX *ctx;
 	{
 	fprintf(fp,"%4ld items in the session cache\n",
 		SSL_CTX_sess_number(ctx));
-	fprintf(fp,"%4ld client connects (SSL_connect())\n",
+	fprintf(fp,"%4d client connects (SSL_connect())\n",
 		SSL_CTX_sess_connect(ctx));
-	fprintf(fp,"%4ld client connects that finished\n",
+	fprintf(fp,"%4d client connects that finished\n",
 		SSL_CTX_sess_connect_good(ctx));
-	fprintf(fp,"%4ld server connects (SSL_accept())\n",
+	fprintf(fp,"%4d server connects (SSL_accept())\n",
 		SSL_CTX_sess_accept(ctx));
-	fprintf(fp,"%4ld server connects that finished\n",
+	fprintf(fp,"%4d server connects that finished\n",
 		SSL_CTX_sess_accept_good(ctx));
-	fprintf(fp,"%4ld session cache hits\n",SSL_CTX_sess_hits(ctx));
-	fprintf(fp,"%4ld session cache misses\n",SSL_CTX_sess_misses(ctx));
-	fprintf(fp,"%4ld session cache timeouts\n",SSL_CTX_sess_timeouts(ctx));
+	fprintf(fp,"%4d session cache hits\n",SSL_CTX_sess_hits(ctx));
+	fprintf(fp,"%4d session cache misses\n",SSL_CTX_sess_misses(ctx));
+	fprintf(fp,"%4d session cache timeouts\n",SSL_CTX_sess_timeouts(ctx));
 	}
 
 static int sv_body(hostname, s)
