@@ -238,7 +238,7 @@ char **argv;
 
 	} else if( strcmp(*argv,"-verify") == 0) {
 
-	    verify=SSL_VERIFY_PEER;
+	    verify=SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE;
 	    if (--argc < 1) goto bad;
 	    verify_depth=atoi(*(++argv));
 	    fprintf(stderr,"verify depth is %d\n",verify_depth);
