@@ -260,7 +260,7 @@ char **argv;
 #define	D_CFB_RC2	10
 #define	D_CBC_RC2	11
 	double d,results[ALGOR_NUM][SIZE_NUM];
-	static int lengths[SIZE_NUM]={8,64,256,1024,8192};
+	static int lengths[SIZE_NUM]={8,64,256,1024,8*1024};
 	long c[ALGOR_NUM][SIZE_NUM];
 	static char *names[ALGOR_NUM]={
 		"md2","md5","sha","sha1","rc4",
