@@ -218,12 +218,13 @@ char *argv[];
 		(strcmp(argv[0],"exit") == 0) ||
 		(strcmp(argv[0],"bye") == 0))
 		{
-		ret=-1;
+		ret= -1;
 		goto end;
 		}
 	else
 		{
-		fprintf(stderr,"bad command, valid commands are");
+		fprintf(stderr,"'%s' is a bad command, valid commands are",
+			argv[0]);
 		i=0;
 		fp=functions;
 		tp=0;
