@@ -89,7 +89,7 @@ static char *usage[]={
 " -startdate      - notBefore field\n",
 " -enddate        - notAfter field\n",
 " -dates          - both Before and After dates\n",
-" -modulus        - print the private key modulus\n",
+" -modulus        - print the RSA key modulus\n",
 " -fingerprint    - print the certificate fingerprint\n",
 " -noout          - no certificate output\n",
 
@@ -121,6 +121,8 @@ static int sign ();
 static int certify ();
 #endif
 
+static int reqfile=0;
+
 int MAIN(argc, argv)
 int argc;
 char **argv;
@@ -137,13 +139,15 @@ char **argv;
 	char *CAkeyfile=NULL,*str=NULL,*CAserial=NULL;
 	int text=0,serial=0,hash=0,subject=0,issuer=0,startdate=0,enddate=0;
 	int noout=0,sign_flag=0,CA_flag=0,CA_createserial=0;
-	int x509req=0,days=DEF_DAYS,reqfile=0,modulus=0;
+	int x509req=0,days=DEF_DAYS,modulus=0;
 	char **pp;
 	CERTIFICATE_CTX *ctx=NULL;
 	X509_REQ *rq=NULL;
 	RSA *r=NULL;
 	int fingerprint=0;
 
+	reqfile=0;
+
 	apps_startup();
 
 	if (bio_err == NULL)
@@ -348,7 +352,13 @@ bad:
 	                }
 		i=X509_REQ_verify(req,pkey);
 		EVP_PKEY_free(pkey);
-	        if (!i)
+		if (i < 0)
+			{
+			fprintf(stderr,"Signature verification error\n");
+			ERR_print_errors(bio_err);
+			goto end;
+			}
+	        if (i == 0)
 			{
 			fprintf(stderr,"Signature did not match the certificate request\n");
 			goto end;
@@ -726,8 +736,10 @@ int days;
 	/* Must up the references since X509_add_cert thinks it can keep
 	 * the cert for it's very own :-) */
 	x->references++;
-	/* NOTE: this certificate can/should be self signed */
-	if (!X509_cert_verify(ctx,x,callb,NULL))
+
+	/* NOTE: this certificate can/should be self signed, unless it was
+	 * a certificate request in which case it is not. */
+	if (!reqfile && !X509_cert_verify(ctx,x,callb,NULL))
 		goto end;
 
 	X509_NAME_free(x->cert_info->issuer);
