@@ -239,10 +239,10 @@ ASN1_INTEGER *a;
 	if (a->data == NULL)
 		return(0);
 
-	for (i=a->length; i > 0; i--)
+	for (i=0; i<a->length; i++)
 		{
 		r<<=8;
-		r|=(unsigned char)a->data[i-1];
+		r|=(unsigned char)a->data[i];
 		}
 	if (neg) r= -r;
 	return(r);
