@@ -87,7 +87,7 @@ EVP_MD *type;
 			a->parameter->type=V_ASN1_NULL;
 			}
 		ASN1_OBJECT_free(a->algorithm);
-		a->algorithm=OBJ_nid2obj(type->type);
+		a->algorithm=OBJ_nid2obj(type->pkey_type);
 		if (a->algorithm == NULL)
 			{
 			ASN1err(ASN1_F_ASN1_SIGN,ASN1_R_UNKNOWN_OBJECT_TYPE);
