@@ -198,7 +198,7 @@ ASN1_UTCTIME *a;
 
 ASN1_UTCTIME *ASN1_UTCTIME_set(s, t)
 ASN1_UTCTIME *s;
-long t;
+time_t t;
 	{
 	struct tm *ts;
 #if defined(THREADS) && defined(sun)
