@@ -1,5 +1,5 @@
-/* lib/asn1/n_pkey.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
+/* crypto/asn1/n_pkey.c */
+/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
  * 
  * This file is part of an SSL implementation written
@@ -46,7 +46,7 @@
  */
 
 #include <stdio.h>
-#include "crypto.h"
+#include "cryptlib.h"
 #include "rsa.h"
 #include "objects.h"
 #include "asn1_mac.h"
@@ -54,7 +54,7 @@
 #include "x509.h"
 
 
-#ifndef NORC4
+#ifndef NO_RC4
 
 typedef struct netscape_pkey_st
 	{
@@ -145,9 +145,9 @@ int (*cb)();
 
 	if (pkey->private_key->data != NULL)
 		free(pkey->private_key->data);
-	if ((pkey->private_key->data=malloc(l[0])) == NULL)
+	if ((pkey->private_key->data=(unsigned char *)malloc(l[0])) == NULL)
 		{
-		ASN1err(ASN1_F_I2D_NETSCAPE_RSA_2,ERR_R_MALLOC_FAILURE);
+		ASN1err(ASN1_F_I2D_NETSCAPE_RSA,ERR_R_MALLOC_FAILURE);
 		goto err;
 		}
 	zz=pkey->private_key->data;
@@ -155,7 +155,7 @@ int (*cb)();
 
 	if ((os2.data=(unsigned char *)malloc(os2.length)) == NULL)
 		{
-		ASN1err(ASN1_F_I2D_NETSCAPE_RSA_2,ERR_R_MALLOC_FAILURE);
+		ASN1err(ASN1_F_I2D_NETSCAPE_RSA,ERR_R_MALLOC_FAILURE);
 		goto err;
 		}
 	zz=os2.data;
@@ -166,12 +166,13 @@ int (*cb)();
 	i=cb(buf,256,"Enter Private Key password:",1);
 	if (i != 0)
 		{
-		ASN1err(ASN1_F_I2D_NETSCAPE_RSA_2,ASN1_R_BAD_PASSWD_READ);
+		ASN1err(ASN1_F_I2D_NETSCAPE_RSA,ASN1_R_BAD_PASSWD_READ);
 		goto err;
 		}
-	EVP_BytesToKey(EVP_rc4,EVP_md5,NULL,buf,strlen((char *)buf),1,key,NULL);
+	EVP_BytesToKey(EVP_rc4(),EVP_md5(),NULL,buf,
+		strlen((char *)buf),1,key,NULL);
 	memset(buf,0,256);
-	EVP_EncryptInit(&ctx,EVP_rc4,key,NULL);
+	EVP_EncryptInit(&ctx,EVP_rc4(),key,NULL);
 	EVP_EncryptUpdate(&ctx,os2.data,&i,os2.data,os2.length);
 	EVP_EncryptFinal(&ctx,&(os2.data[i]),&j);
 	memset(&ctx,0,sizeof(ctx));
@@ -187,6 +188,7 @@ err:
 	if (os2.data != NULL) free(os2.data);
 	if (alg != NULL) X509_ALGOR_free(alg);
 	if (pkey != NULL) NETSCAPE_PKEY_free(pkey);
+	r=r;
 	return(ret);
 	}
 
@@ -258,9 +260,10 @@ int (*cb)();
 		goto err;
 		}
 
-	EVP_BytesToKey(EVP_rc4,EVP_md5,NULL,buf,strlen((char *)buf),1,key,NULL);
+	EVP_BytesToKey(EVP_rc4(),EVP_md5(),NULL,buf,
+		strlen((char *)buf),1,key,NULL);
 	memset(buf,0,256);
-	EVP_DecryptInit(&ctx,EVP_rc4,key,NULL);
+	EVP_DecryptInit(&ctx,EVP_rc4(),key,NULL);
 	EVP_DecryptUpdate(&ctx,os->data,&i,os->data,os->length);
 	EVP_DecryptFinal(&ctx,&(os->data[i]),&j);
 	memset(&ctx,0,sizeof(ctx));
@@ -346,5 +349,5 @@ NETSCAPE_PKEY *a;
 	free(a);
 	}
 
-#endif /* NORC4 */
+#endif /* NO_RC4 */
 
