@@ -62,7 +62,7 @@ unsigned char **pp;
 	M_ASN1_I2D_vars(a);
 
 	M_ASN1_I2D_len(a->version,i2d_ASN1_INTEGER);
-	M_ASN1_I2D_len_SET(a->md_algs,i2d_ASN1_OBJECT);
+	M_ASN1_I2D_len_SET(a->md_algs,i2d_X509_ALGOR);
 	M_ASN1_I2D_len(a->contents,i2d_PKCS7);
 	M_ASN1_I2D_len_IMP_set_opt(a->cert,i2d_X509,0);
 	M_ASN1_I2D_len_IMP_set_opt(a->crl,i2d_X509_CRL,1);
@@ -71,7 +71,7 @@ unsigned char **pp;
 	M_ASN1_I2D_seq_total();
 
 	M_ASN1_I2D_put(a->version,i2d_ASN1_INTEGER);
-	M_ASN1_I2D_put_SET(a->md_algs,i2d_ASN1_OBJECT);
+	M_ASN1_I2D_put_SET(a->md_algs,i2d_X509_ALGOR);
 	M_ASN1_I2D_put(a->contents,i2d_PKCS7);
 	M_ASN1_I2D_put_IMP_set_opt(a->cert,i2d_X509,0);
 	M_ASN1_I2D_put_IMP_set_opt(a->crl,i2d_X509_CRL,1);
@@ -90,7 +90,7 @@ long length;
 	M_ASN1_D2I_Init();
 	M_ASN1_D2I_start_sequence();
 	M_ASN1_D2I_get(ret->version,d2i_ASN1_INTEGER);
-	M_ASN1_D2I_get_set(ret->md_algs,d2i_ASN1_OBJECT);
+	M_ASN1_D2I_get_set(ret->md_algs,d2i_X509_ALGOR);
 	M_ASN1_D2I_get(ret->contents,d2i_PKCS7);
 	M_ASN1_D2I_get_IMP_set_opt(ret->cert,d2i_X509,0);
 	M_ASN1_D2I_get_IMP_set_opt(ret->crl,d2i_X509_CRL,1);
@@ -119,7 +119,7 @@ PKCS7_SIGNED *a;
 	{
 	if (a == NULL) return;
 	ASN1_INTEGER_free(a->version);
-	sk_pop_free(a->md_algs,ASN1_OBJECT_free);
+	sk_pop_free(a->md_algs,X509_ALGOR_free);
 	PKCS7_free(a->contents);
 	sk_pop_free(a->cert,X509_free);
 	sk_pop_free(a->crl,X509_CRL_free);
