@@ -63,7 +63,7 @@ unsigned char **pp;
 
 	M_ASN1_I2D_len(a->version,i2d_ASN1_INTEGER);
 	M_ASN1_I2D_len(a->issuer_and_serial,i2d_PKCS7_ISSUER_AND_SERIAL);
-	M_ASN1_I2D_len(a->digest_alg,i2d_ASN1_OBJECT);
+	M_ASN1_I2D_len(a->digest_alg,i2d_X509_ALGOR);
 	M_ASN1_I2D_len_IMP_set_opt(a->auth_attr,i2d_X509_ATTRIBUTE,0);
 	M_ASN1_I2D_len(a->digest_enc_alg,i2d_X509_ALGOR);
 	M_ASN1_I2D_len(a->enc_digest,i2d_ASN1_OCTET_STRING);
@@ -73,7 +73,7 @@ unsigned char **pp;
 
 	M_ASN1_I2D_put(a->version,i2d_ASN1_INTEGER);
 	M_ASN1_I2D_put(a->issuer_and_serial,i2d_PKCS7_ISSUER_AND_SERIAL);
-	M_ASN1_I2D_put(a->digest_alg,i2d_ASN1_OBJECT);
+	M_ASN1_I2D_put(a->digest_alg,i2d_X509_ALGOR);
 	M_ASN1_I2D_put_IMP_set_opt(a->auth_attr,i2d_X509_ATTRIBUTE,0);
 	M_ASN1_I2D_put(a->digest_enc_alg,i2d_X509_ALGOR);
 	M_ASN1_I2D_put(a->enc_digest,i2d_ASN1_OCTET_STRING);
@@ -93,7 +93,7 @@ long length;
 	M_ASN1_D2I_start_sequence();
 	M_ASN1_D2I_get(ret->version,d2i_ASN1_INTEGER);
 	M_ASN1_D2I_get(ret->issuer_and_serial,d2i_PKCS7_ISSUER_AND_SERIAL);
-	M_ASN1_D2I_get(ret->digest_alg,d2i_ASN1_OBJECT);
+	M_ASN1_D2I_get(ret->digest_alg,d2i_X509_ALGOR);
 	M_ASN1_D2I_get_IMP_set_opt(ret->auth_attr,d2i_X509_ATTRIBUTE,0);
 	M_ASN1_D2I_get(ret->digest_enc_alg,d2i_X509_ALGOR);
 	M_ASN1_D2I_get(ret->enc_digest,d2i_ASN1_OCTET_STRING);
@@ -110,7 +110,7 @@ PKCS7_SIGNER_INFO *PKCS7_SIGNER_INFO_new()
 	M_ASN1_New_Malloc(ret,PKCS7_SIGNER_INFO);
 	M_ASN1_New(ret->version,ASN1_INTEGER_new);
 	M_ASN1_New(ret->issuer_and_serial,PKCS7_ISSUER_AND_SERIAL_new);
-	M_ASN1_New(ret->digest_alg,ASN1_OBJECT_new);
+	M_ASN1_New(ret->digest_alg,X509_ALGOR_new);
 	ret->auth_attr=NULL;
 	M_ASN1_New(ret->digest_enc_alg,X509_ALGOR_new);
 	M_ASN1_New(ret->enc_digest,ASN1_OCTET_STRING_new);
@@ -125,7 +125,7 @@ PKCS7_SIGNER_INFO *a;
 	if (a == NULL) return;
 	ASN1_INTEGER_free(a->version);
 	PKCS7_ISSUER_AND_SERIAL_free(a->issuer_and_serial);
-	ASN1_OBJECT_free(a->digest_alg);
+	X509_ALGOR_free(a->digest_alg);
 	sk_pop_free(a->auth_attr,X509_ATTRIBUTE_free);
 	X509_ALGOR_free(a->digest_enc_alg);
 	ASN1_OCTET_STRING_free(a->enc_digest);
