@@ -103,13 +103,16 @@ X509 *X509_KEY_new()
 	M_ASN1_New_Error(ASN1_F_X509_NEW);
 	}
 
-void X509_free(a)
+void X509_KEY_free(a)
 X509 *a;
 	{
+	int i;
+
 	if (a == NULL) return;
-	CS_BEGIN
-	if (--a->references > 0) return;
-	CS_END
+
+	i=CRYPTO_add_lock(&a->references,-1,CRYPTO_LOCK_X509_KEY);
+	if (i > 0) return;
+
 	X509_CINF_free(a->cert_info);
 	X509_ALGOR_free(a->sig_alg);
 	ASN1_BIT_STRING_free(a->signature);
