@@ -155,7 +155,7 @@ X509 *x;
 			goto err;
 		RSA_print(bp,rsa,16);
 		break;
-	case NID_dss:
+	case NID_dsa:
 	case NID_dsaWithSHA:
 	case NID_dsaWithSHA1:
 		if (d2i_DSAPublicKey(&dsa,(unsigned char **)&s, len) == NULL)
