@@ -72,15 +72,12 @@ X509_INFO *X509_INFO_new()
 void X509_INFO_free(x)
 X509_INFO *x;
 	{
+	int i;
+
 	if (x == NULL) return;
 
-	CRYPTO_w_lock(CRYPTO_LOCK_X509_INFO);
-	if (--x->references > 0)
-		{
-		CRYPTO_w_unlock(CRYPTO_LOCK_X509_INFO);
-		return;
-		}
-	CRYPTO_w_unlock(CRYPTO_LOCK_X509_INFO);
+	i=CRYPTO_add(&x->references,-1,CRYPTO_LOCK_X509_INFO);
+	if (i > 0) return;
 
 	if (x->x509 != NULL) X509_free(x->x509);
 	if (x->crl != NULL) X509_CRL_free(x->crl);
