@@ -97,6 +97,7 @@ unsigned char **pp;
 	int set=0,r,ret=0;
 	int i;
 	unsigned char *p;
+	int size=0;
 
 	if (a == NULL) return(0);
 	for (i=0; i<sk_num(a); i++)
@@ -105,24 +106,27 @@ unsigned char **pp;
 		if (fe == NULL)
 			{
 			fe=ne;
-			fe->size=0;
+			size=0;
 			}
 
 		if (ne->set != set)
 			{
-			ret+=ASN1_object_size(1,fe->size,V_ASN1_SET);
+			ret+=ASN1_object_size(1,size,V_ASN1_SET);
+			fe->size=size;
 			fe=ne;
-			fe->size=0;
+			size=0;
 			set=ne->set;
 			}
-		fe->size+=i2d_X509_NAME_ENTRY(ne,NULL);
+		size+=i2d_X509_NAME_ENTRY(ne,NULL);
 		}
 
-	ret+=ASN1_object_size(1,fe->size,V_ASN1_SET);
+	ret+=ASN1_object_size(1,size,V_ASN1_SET);
+	if (fe != NULL)
+		fe->size=size;
 
 	M_ASN1_I2D_seq_total();
 
-	set=-1;
+	set= -1;
 	for (i=0; i<sk_num(a); i++)
 		{
 		ne=(X509_NAME_ENTRY *)sk_value(a,i);
@@ -147,6 +151,13 @@ long length;
 	int index=0;
 	M_ASN1_D2I_vars(a,X509_NAME *,X509_NAME_new);
 
+	if (sk_num(ret) > 0)
+		{
+		while (sk_num(ret) > 0)
+			X509_NAME_ENTRY_free((X509_NAME_ENTRY *)
+				sk_pop(ret));
+		}
+
 	M_ASN1_D2I_Init();
 	M_ASN1_D2I_start_sequence();
 	for (;;)
