@@ -115,15 +115,12 @@ X509 *X509_new()
 void X509_free(a)
 X509 *a;
 	{
+	int i;
+
 	if (a == NULL) return;
 
-	CRYPTO_w_lock(CRYPTO_LOCK_X509);
-	if (--a->references > 0)
-		{
-		CRYPTO_w_unlock(CRYPTO_LOCK_X509);
-		return;
-		}
-	CRYPTO_w_unlock(CRYPTO_LOCK_X509);
+	i=CRYPTO_add(&a->references,-1,CRYPTO_LOCK_X509);
+	if (i > 0) return;
 
 	X509_CINF_free(a->cert_info);
 	X509_ALGOR_free(a->sig_alg);
