@@ -50,6 +50,7 @@
 #include "bn.h"
 #include "bn_lcl.h"
 
+/* The old slow way */
 #ifdef undef
 int BN_div(dv, rem, m, d,ctx)
 BIGNUM *dv;
@@ -67,7 +68,7 @@ BN_CTX *ctx;
 		return(0);
 		}
 
-	if (BN_cmp(m,d) < 0)
+	if (BN_jjcmp(m,d) < 0)
 		{
 		if (rem != NULL)
 			{ if (BN_copy(rem,m) == NULL) return(0); }
@@ -98,7 +99,7 @@ BN_CTX *ctx;
 			dv->d[0]|=1;
 			if (!BN_sub(rem,rem,D)) return(0);
 			}
-/* CAN IMPROVE */
+/* CAN IMPROVE (and have now :=) */
 		if (!BN_rshift1(D,D)) return(0);
 		}
 	dv->neg=m->neg^d->neg;
@@ -126,6 +127,7 @@ BN_CTX *ctx;
 		return(0);
 		}
 
+/*	if (BN_ucmp(num,div) < 0)*/
 	if (BN_cmp(num,div) < 0)
 		{
 		if (rm != NULL)
@@ -167,15 +169,15 @@ BN_CTX *ctx;
 	wnump= &(snum->d[num_n-1]);
 
 	/* Setup to 'res' */
-	res->neg=0;
+	res->neg= (num->neg^div->neg);
 	res->top=loop;
-	bn_expand(res,(loop+1)*BN_BITS2);
+	if (!bn_expand(res,(loop+1)*BN_BITS2)) goto err;
 	resp= &(res->d[loop-1]);
 
 	/* space for temp */
-	bn_expand(tmp,(div_n+1)*BN_BITS2);
+	if (!bn_expand(tmp,(div_n+1)*BN_BITS2)) goto err;
 
-	if (BN_cmp(&wnum,sdiv) >= 0)
+	if (BN_ucmp(&wnum,sdiv) >= 0)
 		{
 		BN_sub(&wnum,&wnum,sdiv);
 		*resp=1;
@@ -261,8 +263,13 @@ BN_CTX *ctx;
 		wnump--;
 		}
 	if (rm != NULL)
+		{
 		BN_rshift(rm,snum,norm_shift);
+		rm->neg=num->neg;
+		}
 	return(1);
+err:
+	return(0);
 	}
 
 #endif
