@@ -64,11 +64,32 @@
 #include "crypto.h"
 #include "date.h"
 
+/* real #defines in crypto.h, keep these upto date */
+static char* lock_names[CRYPTO_NUM_LOCKS] =
+	{
+	"<<ERROR>>",
+	"err",
+	"err_hash",
+	"x509",
+	"x509_info",
+	"x509_pkey",
+	"x509_crl",
+	"x509_req",
+	"dsa",
+	"rsa",
+	"evp_pkey",
+	"certificate_ctx",
+	"ssl_ctx",
+	"ssl_cert",
+	"ssl_session",
+	"ssl",
+	};
+
 char *SSLeay_version(t)
 int t;
 	{
 	if (t == SSLEAY_VERSION)
-		return("SSLeay 0.6.1 12-Jul-1996");
+		return("SSLeay 0.6.3 06-Aug-1996");
 	if (t == SSLEAY_OPTIONS)
 		{
 		static char buf[100];
@@ -115,24 +136,39 @@ unsigned long SSLeay()
 	}
 
 #ifndef NOPROTO
-static void (MS_FAR *locking_callback)(int mode,int type)=NULL;
+static void (MS_FAR *locking_callback)(int mode,int type,
+	char *file,int line)=NULL;
+static int (MS_FAR *add_lock_callback)(int *pointer,int amount,
+	int type,char *file,int line)=NULL;
 static unsigned long (MS_FAR *id_callback)(void)=NULL;
 #else
 static void (MS_FAR *locking_callback)()=NULL;
+static int (MS_FAR *add_lock_callback)()=NULL;
 static unsigned long (MS_FAR *id_callback)()=NULL;
 #endif
 
-void (*CRYPTO_get_locking_callback())(P_I_I)
+void (*CRYPTO_get_locking_callback())(P_I_I_P_I)
 	{
 	return(locking_callback);
 	}
 
+int (*CRYPTO_get_add_lock_callback())(P_IP_I_I_P_I)
+	{
+	return(add_lock_callback);
+	}
+
 void CRYPTO_set_locking_callback(func)
-void (*func)(P_I_I);
+void (*func)(P_I_I_P_I);
 	{
 	locking_callback=func;
 	}
 
+void CRYPTO_set_add_lock_callback(func)
+int (*func)(P_IP_I_I_P_I);
+	{
+	add_lock_callback=func;
+	}
+
 unsigned long (*CRYPTO_get_id_callback(P_V))(P_V)
 	{
 	return(id_callback);
@@ -165,12 +201,86 @@ unsigned long CRYPTO_thread_id()
 	return(ret);
 	}
 
-void CRYPTO_lock(mode,type)
+void CRYPTO_lock(mode,type,file,line)
 int mode;
 int type;
+char *file;
+int line;
 	{
-/*	fprintf(stderr,"mode= %02X  type= %d\n",mode,type);*/
+#ifdef LOCK_DEBUG
+		{
+		char *rw_text,*operation_text;
 
+		if (mode & CRYPTO_LOCK)
+			operation_text="lock  ";
+		else if (mode & CRYPTO_UNLOCK)
+			operation_text="unlock";
+		else
+			operation_text="ERROR ";
+
+		if (mode & CRYPTO_READ)
+			rw_text="r";
+		else if (mode & CRYPTO_WRITE)
+			rw_text="w";
+		else
+			rw_text="ERROR";
+
+		fprintf(stderr,"lock:%08lx:(%s)%s %-18s %s:%d\n",
+			CRYPTO_thread_id(), rw_text, operation_text,
+			CRYPTO_get_lock_name(type), file, line);
+		}
+#endif
 	if (locking_callback != NULL)
-		locking_callback(mode,type);
+		locking_callback(mode,type,file,line);
+	}
+
+int CRYPTO_add_lock(pointer,amount,type,file,line)
+int *pointer;
+int amount;
+int type;
+char *file;
+int line;
+	{
+	int ret;
+
+	if (add_lock_callback != NULL)
+		{
+#ifdef LOCK_DEBUG
+		int before= *pointer;
+#endif
+
+		ret=add_lock_callback(pointer,amount,type,file,line);
+#ifdef LOCK_DEBUG
+		fprintf(stderr,"ladd:%08lx:%2d+%2d->%2d %-18s %s:%d\n",
+			CRYPTO_thread_id(),
+			before,amount,ret,
+			CRYPTO_get_lock_name(type),
+			file,line);
+#endif
+		*pointer=ret;
+		}
+	else
+		{
+		CRYPTO_lock(CRYPTO_LOCK|CRYPTO_WRITE,type,file,line);
+
+		ret= *pointer+amount;
+#ifdef LOCK_DEBUG
+		fprintf(stderr,"ladd:%08lx:%2d+%2d->%2d %-18s %s:%d\n",
+			CRYPTO_thread_id(),
+			*pointer,amount,ret,
+			CRYPTO_get_lock_name(type),
+			file,line);
+#endif
+		*pointer=ret;
+		CRYPTO_lock(CRYPTO_UNLOCK|CRYPTO_WRITE,type,file,line);
+		}
+	return(ret);
+	}
+
+char *CRYPTO_get_lock_name(type)
+int type;
+	{
+	if ((type < 0) || (type >= CRYPTO_NUM_LOCKS))
+		return("ERROR");
+	return(lock_names[type]);
 	}
