@@ -112,6 +112,7 @@
 #include "asn1/a_hdr.c"
 #include "asn1/a_i2d_fp.c"
 #include "asn1/a_int.c"
+#include "asn1/a_bool.c"
 #include "asn1/a_object.c"
 #include "asn1/a_octet.c"
 #include "asn1/a_print.c"
@@ -152,6 +153,7 @@
 #include "asn1/t_x509.c"
 #include "asn1/x_algor.c"
 #include "asn1/x_attrib.c"
+#include "asn1/x_exten.c"
 #include "asn1/x_cinf.c"
 #include "asn1/x_crl.c"
 #include "asn1/x_info.c"
@@ -208,6 +210,7 @@
 #include "des/enc_writ.c"
 #include "des/fcrypt.c"
 #include "des/ncbc_enc.c"
+#include "des/xcbc_enc.c"
 #include "des/ofb64ede.c"
 #include "des/ofb64enc.c"
 #include "des/ofb_enc.c"
@@ -234,6 +237,7 @@
 #include "evp/digest.c"
 #include "evp/e_cbc_3d.c"
 #include "evp/e_cbc_d.c"
+#include "evp/e_xcbc_d.c"
 #include "evp/e_cbc_i.c"
 #include "evp/e_cbc_r2.c"
 #include "evp/e_cfb_3d.c"
