@@ -53,7 +53,7 @@
 #include "dsa.h"
 #include "asn1.h"
 
-char *DSA_version="\0DSA part of SSLeay 0.6.1 12-Jul-1996";
+char *DSA_version="\0DSA part of SSLeay 0.6.3 06-Aug-1996";
 
 DSA *DSA_new()
 	{
@@ -82,15 +82,12 @@ DSA *DSA_new()
 void DSA_free(r)
 DSA *r;
 	{
+	int i;
+
 	if (r == NULL) return;
 
-	CRYPTO_w_lock(CRYPTO_LOCK_DSA);
-	if (--r->references > 0)
-		{
-		CRYPTO_w_unlock(CRYPTO_LOCK_DSA);
-		return;
-		}
-	CRYPTO_w_unlock(CRYPTO_LOCK_DSA);
+	i=CRYPTO_add(&r->references,-1,CRYPTO_LOCK_DSA);
+	if (i > 0) return;
 
 	if (r->x != NULL) BN_clear_free(r->x);
 	if (r->y != NULL) BN_clear_free(r->y);
