@@ -124,15 +124,14 @@ void ERR_load_ERR_strings()
 	if (init)
 		{
 		CRYPTO_w_lock(CRYPTO_LOCK_ERR);
-		if(!init)
+		if (init == 0)
 			{
 			CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
 			return;
 			}
+		init=0;
 		CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
 
-
-		init=0;
 		ERR_load_strings(0,ERR_str_libraries);
 		ERR_load_strings(0,ERR_str_reasons);
 		ERR_load_strings(ERR_LIB_SYS,ERR_str_functs);
@@ -143,22 +142,27 @@ void ERR_load_strings(lib,str)
 int lib;
 ERR_STRING_DATA *str;
 	{
-	CRYPTO_w_lock(CRYPTO_LOCK_ERR_HASH);
-
 	if (error_hash == NULL)
 		{
+		CRYPTO_w_lock(CRYPTO_LOCK_ERR_HASH);
 		error_hash=lh_new(err_hash,err_cmp);
-		if (error_hash == NULL) return;
+		if (error_hash == NULL)
+			{
+			CRYPTO_w_unlock(CRYPTO_LOCK_ERR_HASH);
+			return;
+			}
+		CRYPTO_w_unlock(CRYPTO_LOCK_ERR_HASH);
+
 		ERR_load_ERR_strings();
 		}
 
+	CRYPTO_w_lock(CRYPTO_LOCK_ERR_HASH);
 	while (str->error)
 		{
 		str->error|=ERR_PACK(lib,0,0);
 		lh_insert(error_hash,(char *)str);
 		str++;
 		}
-
 	CRYPTO_w_unlock(CRYPTO_LOCK_ERR_HASH);
 	}
 
@@ -182,9 +186,8 @@ int line;
 	{
 	ERR_STATE *es;
 
-	CRYPTO_w_lock(CRYPTO_LOCK_ERR);
-
 	es=ERR_get_state();
+
 	es->top=(es->top+1)%ERR_NUM_ERRORS;
 	if (es->top == es->bottom)
 		es->bottom=(es->bottom+1)%ERR_NUM_ERRORS;
@@ -192,7 +195,6 @@ int line;
 	es->err_file[es->top]=file;
 	es->err_line[es->top]=line;
 
-	CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
 	}
 
 void ERR_clear_error()
@@ -200,9 +202,8 @@ void ERR_clear_error()
 	int i;
 	ERR_STATE *es;
 
-	CRYPTO_w_lock(CRYPTO_LOCK_ERR);
-
 	es=ERR_get_state();
+
 	for (i=0; i<ERR_NUM_ERRORS; i++)
 		{
 		es->err_buffer[i]=0;
@@ -210,8 +211,6 @@ void ERR_clear_error()
 		es->err_line[i]= -1;
 		}
 	es->top=es->bottom=0;
-
-	CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
 	}
 
 unsigned long ERR_peek_error()
@@ -219,14 +218,13 @@ unsigned long ERR_peek_error()
 	int i;
 	ERR_STATE *es;
 
-	CRYPTO_r_lock(CRYPTO_LOCK_ERR);
-
+	/* should be fine since only one thread should ever be playing
+	 * with the value returned from this call */
 	es=ERR_get_state();
 
 	if (es->bottom == es->top) return(0);
 	i=(es->bottom+1)%ERR_NUM_ERRORS;
 
-	CRYPTO_r_unlock(CRYPTO_LOCK_ERR);
 
 	return(es->err_buffer[i]);
 	}
@@ -237,16 +235,14 @@ unsigned long ERR_get_error()
 	unsigned long ret;
 	ERR_STATE *es;
 
-	CRYPTO_w_lock(CRYPTO_LOCK_ERR);
-
 	es=ERR_get_state();
+
 	if (es->bottom == es->top) return(0);
 	i=(es->bottom+1)%ERR_NUM_ERRORS;
 	es->bottom=i;
 	ret=es->err_buffer[i];
 	es->err_buffer[i]=0;
 
-	CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
 
 	return(ret);
 	}
@@ -258,12 +254,10 @@ int *line;
 	int i=0;
 	ERR_STATE *es;
 
-	CRYPTO_r_lock(CRYPTO_LOCK_ERR);
-
 	es=ERR_get_state();
+
 	if (es->bottom == es->top)
 		{
-		CRYPTO_r_unlock(CRYPTO_LOCK_ERR);
 		return(0);
 		}
 
@@ -271,8 +265,6 @@ int *line;
 	*file=es->err_file[i];
 	*line=es->err_line[i];
 
-	CRYPTO_r_unlock(CRYPTO_LOCK_ERR);
-
 	return(es->err_buffer[i]);
 	}
 
@@ -284,14 +276,10 @@ int *line;
 	unsigned long ret;
 	ERR_STATE *es;
 
-	CRYPTO_r_lock(CRYPTO_LOCK_ERR);
-
 	es=ERR_get_state();
+
 	if (es->bottom == es->top)
-		{
-		CRYPTO_r_unlock(CRYPTO_LOCK_ERR);
 		return(0);
-		}
 
 	i=(es->bottom+1)%ERR_NUM_ERRORS;
 	es->bottom=i;
@@ -302,7 +290,6 @@ int *line;
 	es->err_file[i]="";
 	es->err_line[i]= -1;
 
-	CRYPTO_r_unlock(CRYPTO_LOCK_ERR);
 	return(ret);
 	}
 
@@ -316,8 +303,6 @@ char *ret;
 	unsigned long l,f,r;
 	int i;
 
-	CRYPTO_r_lock(CRYPTO_LOCK_ERR);
-
 	l=ERR_GET_LIB(e);
 	f=ERR_GET_FUNC(e);
 	r=ERR_GET_REASON(e);
@@ -342,8 +327,6 @@ char *ret;
 		sprintf(&(ret[i]),":reason(%ld)",r);
 	else	sprintf(&(ret[i]),":%s",rs);
 
-	CRYPTO_r_unlock(CRYPTO_LOCK_ERR);
-
 	return(ret);
 	}
 
@@ -497,8 +480,10 @@ unsigned long pid;
 	if (pid == 0)
 		pid=(unsigned long)CRYPTO_thread_id();
 	tmp.pid=pid;
+	CRYPTO_w_lock(CRYPTO_LOCK_ERR);
 	p=(ERR_STATE *)lh_retrieve(thread_hash,(char *)&tmp);
 	if (p != NULL) free(p);
+	CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
 	}
 
 ERR_STATE *ERR_get_state()
@@ -509,16 +494,27 @@ ERR_STATE *ERR_get_state()
 
 	pid=(unsigned long)CRYPTO_thread_id();
 
+	CRYPTO_r_lock(CRYPTO_LOCK_ERR);
 	if (thread_hash == NULL)
 		{
-		thread_hash=lh_new(pid_hash,pid_cmp);
-		if (thread_hash == NULL) return(&fallback);
+		CRYPTO_r_unlock(CRYPTO_LOCK_ERR);
+		CRYPTO_w_lock(CRYPTO_LOCK_ERR);
+		if (thread_hash == NULL)
+			{
+			thread_hash=lh_new(pid_hash,pid_cmp);
+			CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
+			if (thread_hash == NULL) return(&fallback);
+			}
+		else
+			CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
 		}
 	else
 		{
 		tmp.pid=pid;
 		ret=(ERR_STATE *)lh_retrieve(thread_hash,(char *)&tmp);
+		CRYPTO_r_unlock(CRYPTO_LOCK_ERR);
 		}
+
 	/* ret == the error state, if NULL, make a new one */
 	if (ret == NULL)
 		{
@@ -527,7 +523,9 @@ ERR_STATE *ERR_get_state()
 		ret->pid=pid;
 		ret->top=0;
 		ret->bottom=0;
+		CRYPTO_w_lock(CRYPTO_LOCK_ERR);
 		tmpp=(ERR_STATE *)lh_insert(thread_hash,(char *)ret);
+		CRYPTO_w_unlock(CRYPTO_LOCK_ERR);
 		if (tmpp != NULL) /* old entry - should not happen */
 			free(tmpp);
 		}
