@@ -49,7 +49,7 @@
 #include "cryptlib.h"
 #include "envelope.h"
 
-char *EVP_version="EVP part of SSLeay 0.6.1 12-Jul-1996";
+char *EVP_version="EVP part of SSLeay 0.6.3 06-Aug-1996";
 
 void EVP_CipherInit(ctx,data,key,iv,enc)
 EVP_CIPHER_CTX *ctx;
@@ -97,7 +97,7 @@ unsigned char *iv;
 	{
 	if (cipher != NULL)
 		ctx->cipher=cipher;
-	cipher->enc_init(ctx,key,iv);
+	ctx->cipher->enc_init(ctx,key,iv);
 	ctx->encrypt=1;
 	ctx->buf_len=0;
 	}
@@ -110,7 +110,7 @@ unsigned char *iv;
 	{
 	if (cipher != NULL)
 		ctx->cipher=cipher;
-	cipher->dec_init(ctx,key,iv);
+	ctx->cipher->dec_init(ctx,key,iv);
 	ctx->encrypt=0;
 	ctx->buf_len=0;
 	}
