@@ -170,9 +170,7 @@ STACK **r;
 			{
 			xi=(X509_INFO *)sk_value(ctx->args,i);
 
-			CRYPTO_w_lock(CRYPTO_LOCK_X509_INFO);
-			xi->references++;
-			CRYPTO_w_unlock(CRYPTO_LOCK_X509_INFO);
+			CRYPTO_add(&xi->references,1,CRYPTO_LOCK_X509_INFO);
 
 			sk_push(sk,(char *)xi);
 			}
