@@ -312,16 +312,17 @@ extern "C" {
 #define SN_netscape			"Netscape"
 #define LN_netscape			"Netscape Communications Corp."
 #define NID_netscape			57
-/* #define OBJ_netscape			2L,16L,840L,113730L */
-#define OBJ_netscape			2L,16L,840L,11370L
+#define OBJ_netscape			2L,16L,840L,1L,113730L
 
-#define LN_netscape_gif2		"ns-gif-2"
-#define NID_netscape_gif2		58
-#define OBJ_netscape_gif2		2L,16L,840L,11370L,1L,2L
+#define SN_netscape_cert_extension	"nsCertExt"
+#define LN_netscape_cert_extension	"Netscape Certificate Extension"
+#define NID_netscape_cert_extension	58
+#define OBJ_netscape_cert_extension	OBJ_netscape,1L
 
-#define LN_netscape_gif3		"ns-gif-3"
-#define NID_netscape_gif3		59
-#define OBJ_netscape_gif3		2L,16L,840L,11370L,1L,3L
+#define SN_netscape_data_type		"nsDataType"
+#define LN_netscape_data_type		"Netscape Data Type"
+#define NID_netscape_data_type		59
+#define OBJ_netscape_data_type		OBJ_netscape,2L
 
 #define SN_des_ede_cfb			"DES-EDE-CFB"
 #define LN_des_ede_cfb			"des-ede-cfb"
@@ -350,16 +351,17 @@ extern "C" {
 #define SN_sha1WithRSAEncryption	"RSA-SHA1"
 #define LN_sha1WithRSAEncryption	"sha1WithRSAEncryption"
 #define NID_sha1WithRSAEncryption	65
+#define OBJ_sha1WithRSAEncryption	OBJ_pkcs,1L,5L
 
-#define SN_dsaWithSHA			"DSS-SHA"
-#define LN_dsaWithSHA			"shaWithDSSEncryption"
+#define SN_dsaWithSHA			"DSA-SHA"
+#define LN_dsaWithSHA			"dsaWithSHA"
 #define NID_dsaWithSHA			66
-#define OBJ_dsaWithSHA			OBJ_algorithm,12L
+#define OBJ_dsaWithSHA			OBJ_algorithm,13L
 
-#define SN_dss				"DSS"
-#define LN_dss				"dssEncryption"
-#define NID_dss				67
-#define OBJ_dss				OBJ_algorithm,13L
+#define SN_dsa				"DSA"
+#define LN_dsa				"dsaEncryption"
+#define NID_dsa				67
+#define OBJ_dsa				OBJ_algorithm,12L
 
 /* proposed by microsoft to RSA */
 #define LN_pbeWithSHA1AndRC2_CBC	"pbeWithSHA1AndRC2-CBC"
@@ -372,9 +374,58 @@ extern "C" {
 #define OBJ_pbeWithSHA1AndRC4		OBJ_pkcs,5L,12L 
 
 #define SN_dsaWithSHA1			"DSS-SHA1"
-#define LN_dsaWithSHA1			"sha1WithDSSEncryption"
+#define LN_dsaWithSHA1			"dsaWithSHA1"
 #define NID_dsaWithSHA1			70
 
+#define SN_netscape_cert_type		"nsCertType"
+#define LN_netscape_cert_type		"Netscape Cert Type"
+#define NID_netscape_cert_type		71
+#define OBJ_netscape_cert_type		OBJ_netscape_cert_extension,1L
+
+#define SN_netscape_base_url		"nsBaseUrl"
+#define LN_netscape_base_url		"Netscape Base Url"
+#define NID_netscape_base_url		72
+#define OBJ_netscape_base_url		OBJ_netscape_cert_extension,2L
+
+#define SN_netscape_revocation_url	"nsRevocationUrl"
+#define LN_netscape_revocation_url	"Netscape Revocation Url"
+#define NID_netscape_revocation_url	73
+#define OBJ_netscape_revocation_url	OBJ_netscape_cert_extension,3L
+
+#define SN_netscape_ca_revocation_url	"nsCaRevocationUrl"
+#define LN_netscape_ca_revocation_url	"Netscape CA Revocation Url"
+#define NID_netscape_ca_revocation_url	74
+#define OBJ_netscape_ca_revocation_url	OBJ_netscape_cert_extension,4L
+
+#define SN_netscape_renewal_url		"nsRenewalUrl"
+#define LN_netscape_renewal_url		"Netscape Renewal Url"
+#define NID_netscape_renewal_url	75
+#define OBJ_netscape_renewal_url	OBJ_netscape_cert_extension,7L
+
+#define SN_netscape_ca_policy_url	"nsCaPolicyUrl"
+#define LN_netscape_ca_policy_url	"Netscape CA Policy Url"
+#define NID_netscape_ca_policy_url	76
+#define OBJ_netscape_ca_policy_url	OBJ_netscape_cert_extension,8L
+
+#define SN_netscape_ssl_server_name	"nsSslServerName"
+#define LN_netscape_ssl_server_name	"Netscape SSL Server Name"
+#define NID_netscape_ssl_server_name	77
+#define OBJ_netscape_ssl_server_name	OBJ_netscape_cert_extension,12L
+
+#define SN_netscape_comment		"nsComment"
+#define LN_netscape_comment		"Netscape Comment"
+#define NID_netscape_comment		78
+#define OBJ_netscape_comment		OBJ_netscape_cert_extension,13L
+
+#define SN_netscape_cert_sequence	"nsCertSequence"
+#define LN_netscape_cert_sequence	"Netscape Certificate Sequence"
+#define NID_netscape_cert_sequence	79
+#define OBJ_netscape_cert_sequence	OBJ_netscape_data_type,5L
+
+#define SN_desx_cbc			"DESX-CBC"
+#define LN_desx_cbc			"desx-cbc"
+#define NID_desx_cbc			80
+
 #include "asn1.h"
 
 #ifndef NOPROTO
