@@ -133,6 +133,8 @@ start:
 			d2i=(char *(*)())d2i_RSAPrivateKey;
 			if (xi->x_pkey != NULL) X509_PKEY_free(xi->x_pkey);
 			xi->x_pkey=X509_PKEY_new();
+			if ((xi->x_pkey->dec_pkey=EVP_PKEY_new()) == NULL)
+				goto err;
 			xi->x_pkey->dec_pkey->type=EVP_PKEY_RSA;
 			pp=(char **)&(xi->x_pkey->dec_pkey->pkey.rsa);
 			if ((int)strlen(header) > 10) /* assume encrypted */
@@ -143,6 +145,8 @@ start:
 			d2i=(char *(*)())d2i_DSAPrivateKey;
 			if (xi->x_pkey != NULL) X509_PKEY_free(xi->x_pkey);
 			xi->x_pkey=X509_PKEY_new();
+			if ((xi->x_pkey->dec_pkey=EVP_PKEY_new()) == NULL)
+				goto err;
 			xi->x_pkey->dec_pkey->type=EVP_PKEY_DSA;
 			pp=(char **)&(xi->x_pkey->dec_pkey->pkey.dsa);
 			if ((int)strlen(header) > 10) /* assume encrypted */
