@@ -60,7 +60,7 @@
 #include "des.h"
 #endif
 
-char *PEM_version="PEM part of SSLeay 0.6.1 12-Jul-1996";
+char *PEM_version="PEM part of SSLeay 0.6.3 06-Aug-1996";
 
 #define MIN_LENGTH	4
 
@@ -92,10 +92,15 @@ int w;
 	return(-1);
 #else
 	int i,j;
+	char *prompt;
+
+	prompt=EVP_get_pw_prompt();
+	if (prompt == NULL)
+		prompt="Enter PEM pass phrase:";
 
 	for (;;)
 		{
-		i=EVP_read_pw_string(buf,num,"Enter PEM pass phrase:",w);
+		i=EVP_read_pw_string(buf,num,prompt,w);
 		if (i != 0)
 			{
 			PEMerr(PEM_F_DEF_CALLBACK,PEM_R_PROBLEMS_GETTING_PASSWORD);
