@@ -65,7 +65,7 @@ EVP_PKEY **pubk;
 int npubk;
 	{
 	unsigned char key[EVP_MAX_KEY_LENGTH];
-	int ret=-1;
+	int ret= -1;
 	int i,j,max=0;
 	char *s=NULL;
 
@@ -77,7 +77,7 @@ int npubk;
 			goto err;
 			}
 		j=RSA_size(pubk[i]->pkey.rsa);
-		if (j > max) max=i;
+		if (j > max) max=j;
 		}
 	s=(char *)malloc(max*2);
 	if (s == NULL)
