@@ -50,7 +50,7 @@
 #include "bn.h"
 #include "rsa.h"
 
-char *RSA_version="RSA part of SSLeay 0.6.1 12-Jul-1996";
+char *RSA_version="RSA part of SSLeay 0.6.3 06-Aug-1996";
 
 RSA *RSA_new()
 	{
@@ -79,15 +79,12 @@ RSA *RSA_new()
 void RSA_free(r)
 RSA *r;
 	{
+	int i;
+
 	if (r == NULL) return;
 
-	CRYPTO_w_lock(CRYPTO_LOCK_RSA);
-	if (--r->references > 0)
-		{
-		CRYPTO_w_unlock(CRYPTO_LOCK_RSA);
-		return;
-		}
-	CRYPTO_w_unlock(CRYPTO_LOCK_RSA);
+	i=CRYPTO_add(&r->references,-1,CRYPTO_LOCK_RSA);
+	if (i > 0) return;
 
 	if (r->n != NULL) BN_clear_free(r->n);
 	if (r->e != NULL) BN_clear_free(r->e);
