@@ -58,7 +58,7 @@
 #undef MIN_NODES
 #define MIN_NODES	4
 
-char *STACK_version="STACK part of SSLeay 0.6.1 12-Jul-1996";
+char *STACK_version="STACK part of SSLeay 0.6.3 06-Aug-1996";
 
 #ifndef NOPROTO
 #define	FP_ICC	(int (*)(const void *,const void *))
@@ -110,9 +110,19 @@ int loc;
 		st->data[st->num]=data;
 	else
 		{
-		memcpy( (char *)&(st->data[loc+1]),
+		int i;
+		char **f,**t;
+
+		f=(char **)st->data;
+		t=(char **)&(st->data[1]);
+		for (i=st->num; i>loc; i--)
+			t[i]=f[i];
+			
+#ifdef undef /* no memmove on sunos :-( */
+		memmove( (char *)&(st->data[loc+1]),
 			(char *)&(st->data[loc]),
 			sizeof(char *)*(st->num-loc));
+#endif
 		st->data[loc]=data;
 		}
 	st->num++;
