@@ -89,7 +89,7 @@ EVP_PKEY *pkey;
 	if (pkey->type == EVP_PKEY_RSA)
 		obj=OBJ_nid2obj(NID_rsaEncryption);
 	else if (pkey->type == EVP_PKEY_DSA)
-		obj=OBJ_nid2obj(NID_dss);
+		obj=OBJ_nid2obj(NID_dsa);
 	else
 		{
 		X509err(X509_F_X509_TO_X509_REQ,X509_R_BAD_PUBLIC_KEY_TYPE);
