@@ -78,7 +78,7 @@ static int null_callback();
 void RSA_print_bytes();
 #endif
 
-char *X509_version="X509 part of SSLeay 0.6.1 12-Jul-1996";
+char *X509_version="X509 part of SSLeay 0.6.3 06-Aug-1996";
 
 int X509_add_cert_file(ctx,file, type)
 CERTIFICATE_CTX *ctx;
@@ -282,7 +282,7 @@ int *error;
 		goto end;
 		}
 
-	if (!X509_verify(xs,pkey)) /* EAY */
+	if (X509_verify(xs,pkey) <= 0)
 		{
 		reason=VERIFY_ERR_SIGNATURE_FAILURE;
 		ret=(*cb)(0,xs,xi,depth,reason);
