@@ -53,143 +53,153 @@ extern "C" {
 #endif
 
 #ifndef DEVRANDOM
-#undef DEVRANDOM  /* set this to you 'random' device if you have one */
+#undef DEVRANDOM  /* set this to your 'random' device if you have one */
 #endif
 
 #if defined(NOCONST)
 #define const
 #endif
 
-#if defined(WIN32) || defined(WIN16)
-#ifndef WINDOWS
-#define WINDOWS
-#ifndef MSDOS
-#define MSDOS
-#endif
-#endif
-#endif
-
-#ifdef VC_DEBUG
-#define _DEBUG
-#include <crtdbg.h>
-#endif
 
+/********************************************************************
+ The Microsoft section
+ ********************************************************************/
 /* The following is used becaue of the small stack in some
  * Microsoft operating systems */
 #if defined(WIN16) || defined(MSDOS)
-#define MS_STATIC	static
+#  define MS_STATIC	static
 #else
-#define MS_STATIC
+#  define MS_STATIC
+#endif
+
+#if defined(WIN32) || defined(WIN16)
+#  ifndef WINDOWS
+#    define WINDOWS
+#  endif
+#  ifndef MSDOS
+#    define MSDOS
+#  endif
 #endif
 
 #ifdef WIN16
-#define MS_CALLBACK	_far _loadds
-#define MS_FAR		_far
+#  define MS_CALLBACK	_far _loadds
+#  define MS_FAR	_far
 #else
-#define MS_CALLBACK
-#define MS_FAR
-#endif
-
-#if defined(WINDOWS) || defined(MSDOS)
-#define RFILE	"rand.dat"
-#else
-#define RFILE	".rand"
+#  define MS_CALLBACK
+#  define MS_FAR
 #endif
 
 #if defined(WINDOWS) || defined(MSDOS)
 
-#ifdef WINDOWS
-#include <windows.h>
-#include <stddef.h>
-#include <errno.h>
-#include <string.h>
-#include <malloc.h>
-#endif
-#include <io.h>
-#include <fcntl.h>
+#  ifdef WINDOWS
+#    include <windows.h>
+#    include <stddef.h>
+#    include <errno.h>
+#    include <string.h>
+#    include <malloc.h>
+#  endif
+#  include <io.h>
+#  include <fcntl.h>
 
-#define EXIT(n)		return(n);
-#define LIST_SEPERATOR_CHAR ';'
-#define X_OK	0
-#define W_OK	2
-#define R_OK	4
-#define SSLEAY_CONF	"ssleay.cnf"
-#define NUL_DEV		"nul"
+#  define EXIT(n)		return(n);
+#  define LIST_SEPERATOR_CHAR ';'
+#  define X_OK	0
+#  define W_OK	2
+#  define R_OK	4
+#  define SSLEAY_CONF	"ssleay.cnf"
+#  define NUL_DEV		"nul"
+#  define RFILE		"rand.dat"
 
-#else /* The unix world */
+#else /* The non-microsoft world world */
 
-#include <unistd.h>
+#  ifdef VMS
+#    include <unixlib.h>
+#  else
+#    include <unistd.h>
+#  endif
 
-#define SSLEAY_CONF	"ssleay.conf"
-#define LIST_SEPERATOR_CHAR ':'
-#ifndef MONOLITH
-#define EXIT(n)		exit(n); return(n)
-#else
-#define EXIT(n)		return(n)
-#endif
-#define NUL_DEV		"/dev/null"
+#  define SSLEAY_CONF	"ssleay.conf"
+#  define RFILE		".rand"
+#  define LIST_SEPERATOR_CHAR ':'
+#  ifndef MONOLITH
+#    define EXIT(n)		exit(n); return(n)
+#  else
+#    define EXIT(n)		return(n)
+#  endif
+#  define NUL_DEV		"/dev/null"
 
-#define SSLeay_getpid()	getpid()
+#  define SSLeay_getpid()	getpid()
 
 #endif
 
+/*************/
+
 #ifdef USE_SOCKETS
-#if defined(WINDOWS) || defined(MSDOS)
-/* windows world */
-#include <winsock.h>
+#  if defined(WINDOWS) || defined(MSDOS)
+      /* windows world */
+#    include <winsock.h>
 
-#ifdef NO_SOCK
-#define SSLeay_Write(a,b,c)     (-1)
-#define SSLeay_Read(a,b,c)      (-1)
-#else
-#define SSLeay_Write(a,b,c)     send((a),(b),(c),0)
-#define SSLeay_Read(a,b,c)      recv((a),(b),(c),0)
+#    ifdef NO_SOCK
+#      define SSLeay_Write(a,b,c)     (-1)
+#      define SSLeay_Read(a,b,c)      (-1)
+#    else
+#      define SSLeay_Write(a,b,c)     send((a),(b),(c),0)
+#      define SSLeay_Read(a,b,c)      recv((a),(b),(c),0)
+#    endif
+
+#    define SHUTDOWN(fd)    { shutdown((fd),2); closesocket((fd)); }
+
+#  else
+
+#    ifndef VMS
+      /* unix world */
+#      include <netdb.h>
+#      include <sys/types.h>
+#      include <sys/socket.h>
+#      include <sys/param.h>
+#      include <sys/time.h> /* Needed under linux for FD_XXX */
+#      include <netinet/in.h>
+#    endif
+
+#    if defined(NeXT) || defined(_NEXT_SOURCE)
+#      include <sys/fcntl.h>
+#      include <sys/types.h>
+#    endif
+
+#    ifdef AIX
+#      include <sys/select.h>
+#    endif
+
+#    if defined(sun)
+#      include <sys/filio.h>
+#    else
+#      include <sys/ioctl.h>
+#    endif
+
+#    ifdef VMS
+#      include <unixio.h>
+#    endif
+
+#    define SSLeay_Read(a,b,c)     read((a),(b),(c))
+#    define SSLeay_Write(a,b,c)    write((a),(b),(c))
+#    define SHUTDOWN(fd)    { shutdown((fd),2); close((fd)); }
+#    define INVALID_SOCKET	-1
+#  endif
 #endif
 
-#define SHUTDOWN(fd)    { shutdown((fd),2); closesocket((fd)); }
-
-#else
-
-#ifndef MSDOS
-/* unix world */
-#include <netdb.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/param.h>
-#include <sys/time.h> /* Needed under linux for FD_XXX */
-#include <netinet/in.h>
-
-
-#if defined(NeXT) || defined(_NEXT_SOURCE)
-#include <sys/fcntl.h>
-#include <sys/types.h>
-#endif
-
-#ifdef AIX
-#include <sys/select.h>
-#endif
-
-#if defined(sun)
-#include <sys/filio.h>
-#else
-#include <sys/ioctl.h>
-#endif
-
-#define SSLeay_Read(a,b,c)     read((a),(b),(c))
-#define SSLeay_Write(a,b,c)    write((a),(b),(c))
-#define SHUTDOWN(fd)    { shutdown((fd),2); close((fd)); }
-#define INVALID_SOCKET	-1
-#endif
-#endif
-#endif
+/***********************************************/
 
 #ifndef NOPROTO
 #define P_CC_CC	const void *,const void *
-#define P_I_I	int,int 
-#define P_V	void 
+#define P_I_I		int,int 
+#define P_I_I_P_I	int,int,char *,int
+#define P_IP_I_I_P_I	int *,int,int,char *,int
+#define P_V		void 
 #else
 #define P_CC_CC
 #define P_I_I
+#define P_IP_I_I_P_I
+#define P_I_I_P_I
 #define P_V
 #endif
 
@@ -200,12 +210,6 @@ extern "C" {
 /* do we need to do this for getenv.
  * Just define getenv for use under windows */
 
-#if defined(MSDOS) || defined(_Windows)
-#define LIST_SEPERATOR_CHAR ';'
-#else
-#define LIST_SEPERATOR_CHAR ':'
-#endif
-
 #ifdef WIN16
 /* How to do this needs to be thought out a bit more.... */
 /*char *GETENV(char *);
