@@ -54,13 +54,17 @@ extern "C" {
 
 /* This is more to be used to check the correct DLL is being used
  * in the MS world. */
-#define SSLEAY_VERSION_NUMBER	0x0610	/* Version 0.5.1c would be 0513 */
+#define SSLEAY_VERSION_NUMBER	0x0630	/* Version 0.5.1c would be 0513 */
 
 #define SSLEAY_VERSION		0
 #define SSLEAY_OPTIONS		1
 #define SSLEAY_CFLAGS		2
 #define SSLEAY_BUILT_ON		3
 
+/* When changing the CRYPTO_LOCK_* list, be sure to maintin the text lock
+ * names in cryptlib.c
+ */
+
 #define	CRYPTO_LOCK_ERR			1
 #define	CRYPTO_LOCK_ERR_HASH		2
 #define	CRYPTO_LOCK_X509		3
@@ -86,10 +90,17 @@ extern "C" {
 /* The following stuff is not being used, it was not finished for
  * SSLeay 0.6.0 */
 #ifndef CRYPTO_w_lock
-#define CRYPTO_w_lock(type)	CRYPTO_lock(CRYPTO_LOCK|CRYPTO_WRITE,type)
-#define CRYPTO_w_unlock(type)	CRYPTO_lock(CRYPTO_UNLOCK|CRYPTO_WRITE,type)
-#define CRYPTO_r_lock(type)	CRYPTO_lock(CRYPTO_LOCK|CRYPTO_READ,type)
-#define CRYPTO_r_unlock(type)	CRYPTO_lock(CRYPTO_UNLOCK|CRYPTO_READ,type)
+#define CRYPTO_w_lock(type)	\
+	CRYPTO_lock(CRYPTO_LOCK|CRYPTO_WRITE,type,__FILE__,__LINE__)
+#define CRYPTO_w_unlock(type)	\
+	CRYPTO_lock(CRYPTO_UNLOCK|CRYPTO_WRITE,type,__FILE__,__LINE__)
+#define CRYPTO_r_lock(type)	\
+	CRYPTO_lock(CRYPTO_LOCK|CRYPTO_READ,type,__FILE__,__LINE__)
+#define CRYPTO_r_unlock(type)	\
+	CRYPTO_lock(CRYPTO_UNLOCK|CRYPTO_READ,type,__FILE__,__LINE__)
+#define CRYPTO_add(addr,amount,type)	\
+	CRYPTO_add_lock(addr,amount,type,__FILE__,__LINE__)
+
 #endif
 
 #ifndef NOPROTO
@@ -97,12 +108,20 @@ extern "C" {
 char *SSLeay_version(int type);
 unsigned long SSLeay(void);
 
-void CRYPTO_lock(int mode, int type);
-void CRYPTO_set_locking_callback(void (*func)(int mode,int type));
-void (*CRYPTO_get_locking_callback(void))(int mode,int type);
+void CRYPTO_lock(int mode, int type,char *file,int line);
+void CRYPTO_set_locking_callback(void (*func)(int mode,int type,char *file,
+		int line));
+void (*CRYPTO_get_locking_callback(void))(int mode,int type,char *file,
+		int line);
+void CRYPTO_set_add_lock_callback(int (*func)(int *num,int mount,
+		int type,char *file, int line));
+int (*CRYPTO_get_add_lock_callback(void))(int *num,int mount,
+		int type,char *file,int line);
 void CRYPTO_set_id_callback(unsigned long (*func)(void));
 unsigned long (*CRYPTO_get_id_callback(void))(void);
 unsigned long CRYPTO_thread_id(void);
+char *CRYPTO_get_lock_name(int type);
+int CRYPTO_add_lock(int *pointer,int amount,int type, char *file,int line);
 
 #else
 
@@ -112,9 +131,13 @@ unsigned long SSLeay();
 void CRYPTO_lock();
 void CRYPTO_set_locking_callback();
 void (*CRYPTO_get_locking_callback())();
+void CRYPTO_set_add_lock_callback();
+int (*CRYPTO_get_add_lock_callback())();
 void CRYPTO_set_id_callback();
 unsigned long (*CRYPTO_get_id_callback())();
 unsigned long CRYPTO_thread_id();
+char *CRYPTO_get_lock_name();
+int CRYPTO_add_lock();
 
 #endif
 
