@@ -99,6 +99,13 @@ typedef struct X509_name_entry_st
 
 #define X509_NAME	STACK /* of X509_NAME_ENTRY */
 
+typedef struct X509_extension_st
+	{
+	ASN1_OBJECT *object;
+	int critical;
+	ASN1_OCTET_STRING *value;
+	} X509_EXTENSION;
+
 typedef struct X509_req_info_st
 	{
 	ASN1_INTEGER *version;
@@ -126,9 +133,9 @@ typedef struct x509_cinf_st
 	X509_VAL *validity;
 	X509_NAME *subject;
 	X509_PUBKEY *key;
-	ASN1_BIT_STRING *issuerUID;	/* [ 1 ] optional in v2 */
-	ASN1_BIT_STRING *subjectUID;	/* [ 2 ] optional in v2 */
-	STACK *attributes;		/* [ 3 ] optional in v3 */
+	ASN1_BIT_STRING *issuerUID;		/* [ 1 ] optional in v2 */
+	ASN1_BIT_STRING *subjectUID;		/* [ 2 ] optional in v2 */
+	STACK /* X509_EXTENSION */ *extensions;	/* [ 3 ] optional in v3 */
 	} X509_CINF;
 
 typedef struct x509_st
@@ -148,11 +155,13 @@ typedef struct X509_revoked_st
 
 typedef struct X509_crl_info_st
 	{
+	ASN1_INTEGER *version;
 	X509_ALGOR *sig_alg;
 	X509_NAME *issuer;
 	ASN1_UTCTIME *lastUpdate;
 	ASN1_UTCTIME *nextUpdate;
 	STACK /* X509_REVOKED */ *revoked;
+	STACK /* [0] X509_EXTENSION */ *extensions;
 	} X509_CRL_INFO;
 
 typedef struct X509_crl_st
@@ -306,7 +315,7 @@ typedef struct X509_AUTH_st
 #define X509_CRL_verify(a,r) ASN1_verify((int (*)())i2d_X509_CRL_INFO, \
 	a->sig_alg, a->signature,(char *)a->crl,r)
 
-#define NETSCAPE_SPKI_verify(a,r) ASN1_sign((int (*)())i2d_NETSCAPE_SPKAC, \
+#define NETSCAPE_SPKI_verify(a,r) ASN1_verify((int (*)())i2d_NETSCAPE_SPKAC, \
 		a->sig_algor,a->signature, (char *)a->spkac,r)
 
 #define X509_sign(x,pkey,md) \
@@ -463,6 +472,12 @@ int		i2d_X509_ATTRIBUTE(X509_ATTRIBUTE *a,unsigned char **pp);
 X509_ATTRIBUTE *d2i_X509_ATTRIBUTE(X509_ATTRIBUTE **a,unsigned char **pp,
 			long length);
 
+X509_EXTENSION *X509_EXTENSION_new(void );
+void		X509_EXTENSION_free(X509_EXTENSION *a);
+int		i2d_X509_EXTENSION(X509_EXTENSION *a,unsigned char **pp);
+X509_EXTENSION *d2i_X509_EXTENSION(X509_EXTENSION **a,unsigned char **pp,
+			long length);
+
 X509_NAME_ENTRY *X509_NAME_ENTRY_new(void);
 void		X509_NAME_ENTRY_free(X509_NAME_ENTRY *a);
 int		i2d_X509_NAME_ENTRY(X509_NAME_ENTRY *a,unsigned char **pp);
@@ -614,6 +629,10 @@ X509_ATTRIBUTE *X509_ATTRIBUTE_new();
 void		X509_ATTRIBUTE_free();
 int		i2d_X509_ATTRIBUTE();
 X509_ATTRIBUTE *d2i_X509_ATTRIBUTE();
+X509_EXTENSION *X509_EXTENSION_new();
+void		X509_EXTENSION_free();
+int		i2d_X509_EXTENSION();
+X509_EXTENSION *d2i_X509_EXTENSION();
 X509_NAME_ENTRY *X509_NAME_ENTRY_new();
 void		X509_NAME_ENTRY_free();
 int		i2d_X509_NAME_ENTRY();
