@@ -73,13 +73,10 @@ CERT *ssl_cert_new()
 void ssl_cert_free(c)
 CERT *c;
 	{
-	CRYPTO_w_lock(CRYPTO_LOCK_SSL_CERT);
-	if (--c->references > 0)
-		{
-		CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CERT);
-		return;
-		}
-	CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CERT);
+	int i;
+
+	i=CRYPTO_add(&c->references,-1,CRYPTO_LOCK_SSL_CERT);
+	if (i > 0) return;
 
 	if (c->x509 != NULL) X509_free(c->x509);
 	if (c->privatekey != NULL) EVP_PKEY_free(c->privatekey);
