@@ -116,13 +116,13 @@ SSL *s;
 		case SSL_ST_CONNECT:
 			if ((buf=BUF_MEM_new()) == NULL)
 				{
-				ret=-1;
+				ret= -1;
 				goto end;
 				}
 			if (!BUF_MEM_grow(buf,
 				SSL_MAX_RECORD_LENGTH_3_BYTE_HEADER))
 				{
-				ret=-1;
+				ret= -1;
 				goto end;
 				}
 			if (s->init_buf) free(s->init_buf);
@@ -170,7 +170,7 @@ SSL *s;
 			 * start encrypting, so lets fire it up :-) */
 			if (!s->session->cipher->crypt_init(s,1))
 				{
-				ret=-1;
+				ret= -1;
 				goto end;
 				}
 			s->clear_text=0;
@@ -225,15 +225,12 @@ SSL *s;
 				SSL_add_session(s->ctx,s->session) &&
 				(s->ctx->new_session_cb != NULL))
 				{
-				CRYPTO_w_lock(CRYPTO_LOCK_SSL_SESSION);
-				s->session->references++;
-				CRYPTO_w_unlock(CRYPTO_LOCK_SSL_SESSION);
+				CRYPTO_add(&s->session->references,1,
+					CRYPTO_LOCK_SSL_SESSION);
 
 				if (!s->ctx->new_session_cb(s->session))
 					{
-					CRYPTO_w_lock(CRYPTO_LOCK_SSL_SESSION);
-					s->session->references--;
-					CRYPTO_w_unlock(CRYPTO_LOCK_SSL_SESSION);
+					SSL_SESSION_free(s->session);
 					}
 				}
 
@@ -395,10 +392,12 @@ SSL *s;
 
 	/* hmmm, can we have the problem of the other session with this
 	 * cert, free's it before we increment the reference count. */
+	CRYPTO_w_lock(CRYPTO_LOCK_SSL_SESSION);
 	CRYPTO_w_lock(CRYPTO_LOCK_X509);
 	s->session->peer=s->session->cert->x509;
 	s->session->peer->references++;
-	CRYPTO_w_lock(CRYPTO_LOCK_X509);
+	CRYPTO_w_unlock(CRYPTO_LOCK_X509);
+	CRYPTO_w_unlock(CRYPTO_LOCK_SSL_SESSION);
 
 	/* get conn_id */
 	if (s->conn_id_length != s->state_conn_id_length)
