@@ -61,7 +61,7 @@
 #include "ssl_rsa.h"
 #include "ssl_null.h"
 
-char *SSL_version="SSLeay 0.6.1 12-Jul-1996";
+char *SSL_version="SSLeay 0.6.3 06-Aug-1996";
 
 /* THIS ARRAY MUST BE KEPT ORDERED BY c1, c2 and c3.
  * basically the second last 'value' which is a #define for these 3
@@ -179,7 +179,7 @@ static char *pref_cipher[]={
 #define NUM_OF(x) (sizeof(x) / sizeof(char *))
 static int num_pref_cipher=NUM_OF(pref_cipher);
 
-#if !defined(WIN16) && !defined(_DLL)
+#if !defined(WIN16) && !defined(_DLL) && !defined(VMS)
 FILE *SSL_LOG=stderr;
 FILE *SSL_ERR=stderr;
 #else
@@ -303,17 +303,15 @@ SSL_CTX *ctx;
 	s->session=NULL;
 	s->cert=ctx->default_cert;
 	if (ctx->default_cert != NULL)
-		ctx->default_cert->references++;
+		{
+		CRYPTO_add(&ctx->default_cert->references,1,CRYPTO_LOCK_X509);
+		}
 	s->verify_mode=ctx->default_verify_mode;
 	s->verify_callback=ctx->default_verify_callback;
 	s->ctx=ctx;
 	s->debug=0;
 	s->info_callback=NULL;
-	CS_BEGIN;
-	CRYPTO_w_lock(CRYPTO_LOCK_SSL_CTX);
-	ctx->references++;
-	CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
-	CS_END;
+	CRYPTO_add(&ctx->references,1,CRYPTO_LOCK_SSL_CTX);
 	s->state_ccl=NULL;
 
 	s->verify_result=VERIFY_OK;
@@ -403,7 +401,7 @@ BIO *wbio;
 	{
 	if ((s->rbio != NULL) && (s->rbio != rbio))
 		BIO_free(s->rbio);
-	if ((s->wbio != NULL) && (s->wbio != wbio) && (s->rbio != s->rbio))
+	if ((s->wbio != NULL) && (s->wbio != wbio) && (s->rbio != s->wbio))
 		BIO_free(s->wbio);
 	s->rbio=rbio;
 	s->wbio=wbio;
@@ -420,7 +418,7 @@ SSL *s;
 int SSL_get_fd(s)
 SSL *s;
 	{
-	int ret=-1;
+	int ret= -1;
 	BIO *b;
 
 	b=SSL_get_rbio(s);
@@ -445,12 +443,7 @@ int fd;
 		goto err;
 		}
 	BIO_set_fd(bio,&fd,BIO_NOCLOSE);
-	if (s->rbio != NULL)
-		BIO_free(s->rbio);
-	if ((s->wbio != NULL) && (s->wbio != s->rbio))
-		BIO_free(s->wbio);
-	s->rbio=bio;
-	s->wbio=bio;
+	SSL_set_bio(s,bio,bio);
 	ret=1;
 err:
 	return(ret);
@@ -468,8 +461,7 @@ int fd;
 	if (bio == NULL)
 		{ SSLerr(SSL_F_SSL_SET_WFD,ERR_R_BUF_LIB); goto err; }
 	BIO_set_fd(bio,&fd,BIO_NOCLOSE);
-	if (s->wbio != NULL) BIO_free(s->wbio);
-	s->wbio=bio;
+	SSL_set_bio(s,SSL_get_rbio(s),bio);
 	ret=1;
 err:
 	return(ret);
@@ -490,8 +482,7 @@ int fd;
 		goto err;
 		}
 	BIO_set_fd(bio,&fd,BIO_NOCLOSE);
-	if (s->wbio != NULL) BIO_free(s->wbio);
-	s->rbio=bio;
+	SSL_set_bio(s,bio,SSL_get_wbio(s));
 	ret=1;
 err:
 	return(ret);
@@ -649,8 +640,8 @@ SSL *s;
 	{
 	X509 *r;
 	
+	CRYPTO_w_lock(CRYPTO_LOCK_X509);
 	CRYPTO_r_lock(CRYPTO_LOCK_SSL_SESSION);
-	CRYPTO_w_lock(CRYPTO_LOCK_SSL_CTX);
 
 	if ((s == NULL) || (s->session == NULL))
 		r=NULL;
@@ -661,12 +652,12 @@ SSL *s;
 
 	if (r == NULL)
 		{
-		CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
+		CRYPTO_w_unlock(CRYPTO_LOCK_X509);
 		return(0);
 		}
 
 	r->references++;
-	CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
+	CRYPTO_w_unlock(CRYPTO_LOCK_X509);
 
 	return(r);
 	}
@@ -724,8 +715,7 @@ SSL *s;
 void SSL_copy_session_id(t,f)
 SSL *t,*f;
 	{
-	CRYPTO_r_lock(CRYPTO_LOCK_SSL);
-
+	/* Do we need to to SSL locking? */
 	SSL_set_session(t,SSL_get_session(f));
 
 	CRYPTO_r_lock(CRYPTO_LOCK_X509);
@@ -734,5 +724,4 @@ SSL *t,*f;
 
 	t->cert=f->cert;
 	CRYPTO_r_unlock(CRYPTO_LOCK_X509);
-	CRYPTO_r_unlock(CRYPTO_LOCK_SSL);
 	}
