@@ -152,10 +152,8 @@ X509 *x;
 
 	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
 	if (c->x509 != NULL) X509_free(c->x509);
-	CS_BEGIN;
 	c->x509=x;
-	x->references++;
-	CS_END;
+	CRYPTO_add(&x->references,1,CRYPTO_LOCK_X509);
 	if (c->publickey != NULL) EVP_PKEY_free(c->publickey);
 	pkey=X509_extract_key(x);
 	if (pkey == NULL)
@@ -202,7 +200,7 @@ int type;
 	else if (type == SSL_FILETYPE_PEM)
 		{
 		j=ERR_R_PEM_LIB;
-		x=PEM_read_bio_X509(in,NULL,NULL);
+		x=PEM_read_bio_X509(in,NULL,ssl->ctx->default_passwd_callback);
 		}
 	else
 		{
@@ -273,9 +271,7 @@ RSA *rsa;
 	c->privatekey->type=EVP_PKEY_RSA;
 	c->privatekey->pkey.rsa=rsa;
 
-	CS_BEGIN;
-	rsa->references++;
-	CS_END;
+	CRYPTO_add(&rsa->references,1,CRYPTO_LOCK_RSA);
 	return(1);
 	}
 
@@ -313,7 +309,8 @@ int type;
 	else if (type == SSL_FILETYPE_PEM)
 		{
 		j=ERR_R_PEM_LIB;
-		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,NULL);
+		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,
+			ssl->ctx->default_passwd_callback);
 		}
 	else
 		{
@@ -375,9 +372,7 @@ EVP_PKEY *pkey;
 	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
 	if (c->privatekey) EVP_PKEY_free(c->privatekey);
 	c->privatekey=pkey;
-	CS_BEGIN;
-	pkey->references++;
-	CS_END;
+	CRYPTO_add(&pkey->references,1,CRYPTO_LOCK_EVP_PKEY);
 	return(1);
 	}
 
@@ -410,7 +405,8 @@ int type;
 	if (type == SSL_FILETYPE_PEM)
 		{
 		j=ERR_R_PEM_LIB;
-		pkey=PEM_read_bio_PrivateKey(in,NULL,NULL);
+		pkey=PEM_read_bio_PrivateKey(in,NULL,
+			ssl->ctx->default_passwd_callback);
 		}
 	else
 		{
@@ -468,10 +464,8 @@ X509 *x;
 
 	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
 	if (c->x509 != NULL) X509_free(c->x509);
-	CS_BEGIN;
 	c->x509=x;
-	x->references++;
-	CS_END;
+	CRYPTO_add(&x->references,1,CRYPTO_LOCK_X509);
 	if (c->publickey != NULL) EVP_PKEY_free(c->publickey);
 	pkey=X509_extract_key(x);
 	if (pkey == NULL)
@@ -518,7 +512,7 @@ int type;
 	else if (type == SSL_FILETYPE_PEM)
 		{
 		j=ERR_R_PEM_LIB;
-		x=PEM_read_bio_X509(in,NULL,NULL);
+		x=PEM_read_bio_X509(in,NULL,ctx->default_passwd_callback);
 		}
 	else
 		{
@@ -588,9 +582,7 @@ RSA *rsa;
 	c->privatekey->type=EVP_PKEY_RSA;
 	c->privatekey->pkey.rsa=rsa;
 
-	CS_BEGIN;
-	rsa->references++;
-	CS_END;
+	CRYPTO_add(&rsa->references,1,CRYPTO_LOCK_RSA);
 	return(1);
 	}
 
@@ -628,7 +620,8 @@ int type;
 	else if (type == SSL_FILETYPE_PEM)
 		{
 		j=ERR_R_PEM_LIB;
-		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,NULL);
+		rsa=PEM_read_bio_RSAPrivateKey(in,NULL,
+			ctx->default_passwd_callback);
 		}
 	else
 		{
@@ -689,9 +682,7 @@ EVP_PKEY *pkey;
 	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
 	if (c->privatekey) EVP_PKEY_free(c->privatekey);
 	c->privatekey=pkey;
-	CS_BEGIN;
-	pkey->references++;
-	CS_END;
+	CRYPTO_add(&pkey->references,1,CRYPTO_LOCK_EVP_PKEY);
 	return(1);
 	}
 
@@ -724,7 +715,8 @@ int type;
 	if (type == SSL_FILETYPE_PEM)
 		{
 		j=ERR_R_PEM_LIB;
-		pkey=PEM_read_bio_PrivateKey(in,NULL,NULL);
+		pkey=PEM_read_bio_PrivateKey(in,NULL,
+			ctx->default_passwd_callback);
 		}
 	else
 		{
