@@ -122,10 +122,10 @@ SSL *s;
 		case SSL_ST_BEFORE:
 		case SSL_ST_ACCEPT:
 			if ((buf=BUF_MEM_new()) == NULL)
-				{ ret=-1; goto end; }
+				{ ret= -1; goto end; }
 			if (!BUF_MEM_grow(buf,(int)
 				SSL_MAX_RECORD_LENGTH_3_BYTE_HEADER))
-				{ ret=-1; goto end; }
+				{ ret= -1; goto end; }
 			if (s->init_buf) free(s->init_buf);
 			s->init_buf=(unsigned char *)buf->data;
 			free(buf);
@@ -171,7 +171,7 @@ SSL *s;
 			 * be encrypted. */
 			if (!s->session->cipher->crypt_init(s,0))
 				{
-				ret=-1;
+				ret= -1;
 				goto end;
 				}
 			s->clear_text=0;
@@ -238,15 +238,10 @@ SSL *s;
 				&& SSL_add_session(s->ctx,s->session) &&
 				(s->ctx->new_session_cb != NULL))
 				{
-				CS_BEGIN;
-				s->session->references++;
-				CS_END;
+				CRYPTO_add(&s->session->references,1,
+					CRYPTO_LOCK_SSL_SESSION);
 				if (!s->ctx->new_session_cb(s->session))
-					{
-					CS_BEGIN;
-					s->session->references--;
-					CS_END;
-					}
+					SSL_SESSION_free(s->session);
 				}
 
 			/* flush cache every 32 entries */
@@ -263,7 +258,7 @@ SSL *s;
 
 		default:
 			SSLerr(SSL_F_SSL_ACCEPT,SSL_R_UNKNOWN_STATE);
-			ret=-1;
+			ret= -1;
 			goto end;
 			BREAK;
 			}
@@ -356,14 +351,17 @@ SSL *s;
 		}
 	i=s->cert->private_decrypt(s->cert,s->state_enc,
 		&(p[s->state_clear]),&(p[s->state_clear]));
+	/* j will be 0 or '5' for export ciphers */
 	j=s->session->cipher->enc_bits/8;
 	/* fix so we will not accept clear ciphers when using
 	 * non-export ciphers */
-	if (	((j == 0) && (i != (int)(s->session->cipher->key_size))) ||
+	if (	(i < 0) ||
+		((j == 0) && (i != (int)(s->session->cipher->key_size))) ||
 		((j != 0) && (i != j)))
 		{
-		SSL_TRACE(SSL_ERR,"i=%d enc_bits=%d\n",i,
+		SSL_TRACE(SSL_ERR,"RSA decrypt error - i=%d enc_bits=%d\n",i,
 			s->session->cipher->enc_bits/8);
+		SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,SSL_R_BAD_RSA_DECRYPT);
 		ssl_return_error(s);
 		return(-1);
 		}
@@ -494,7 +492,7 @@ SSL *s;
 
 		/* now for each preference, copy across */
 		i=0;
-		cp=&cc;
+		cp= &cc;
 		for (;;)
 			{
 			str=SSL_get_cipher_list(s,i++);
@@ -586,9 +584,19 @@ SSL *s;
 			if (s->session->cert != NULL)
 				ssl_cert_free(s->session->cert);
 			s->session->cert=s->cert;		
-			CS_BEGIN;
-			s->cert->references++;
-			CS_END;
+			CRYPTO_add(&s->cert->references,1,CRYPTO_LOCK_X509);
+			}
+		else	/* We have a session id-cache hit, if the
+			 * session-id has no certificate listed against
+			 * the 'cert' structure, grab the 'old' one
+			 * listed against the SSL connection */
+			{
+			if (s->session->cert == NULL)
+				{
+				s->session->cert=s->cert;
+				CRYPTO_add(&s->cert->references,1,
+					CRYPTO_LOCK_X509);
+				}
 			}
 
 		if (s->session->cert == NULL)
@@ -757,7 +765,7 @@ SSL *s;
 	{
 	unsigned char *p,*p2,*buf2;
 	unsigned char *ccd;
-	int i,j,ctype,ret=-1;
+	int i,j,ctype,ret= -1;
 	X509 *x509=NULL;
 
 	if (s->state_ccl == NULL)
@@ -894,9 +902,7 @@ SSL *s;
 			if (s->session->peer != NULL)
 				X509_free(s->session->peer);
 			s->session->peer=x509;
-			CS_BEGIN
-			x509->references++;
-			CS_END
+			CRYPTO_add(&x509->references,1,CRYPTO_LOCK_X509);
 			ret=1;
 			goto end;
 			}
